@@ -61,7 +61,7 @@ main (int argc, char **argv)
   CHECK_OPENCL_ERROR_IN ("clCreateContext");
 
   const char *basename = "boxadd";
-  err = poclu_load_program (context, device, basename, 0, 0, 0,
+  err = poclu_load_program (context, device, basename, 0, 0,
 			    NULL, NULL, &program);
   if (err != CL_SUCCESS)
     goto FINISH;
@@ -112,5 +112,9 @@ FINISH:
   CHECK_CL_ERROR (clReleaseContext (context));
   CHECK_CL_ERROR (clUnloadPlatformCompiler (platform));
 
+  free (srcA);
+  free (srcB);
+  free (dst);
+
   return err;
 }
