@@ -48,7 +48,7 @@ main (int argc, char **argv)
 {
   cl_uint *srcA, *srcB;
   cl_uint *dst;
-  int i, err, spir, spirv, poclbin;
+  int i, err, spirv, poclbin;
 
   cl_context context = NULL;
   cl_device_id device = NULL;
@@ -61,13 +61,12 @@ main (int argc, char **argv)
   err = poclu_get_any_device2 (&context, &device, &queue, &platform);
   CHECK_OPENCL_ERROR_IN ("clCreateContext");
 
-  spir = (argc > 1 && argv[1][0] == 's');
   spirv = (argc > 1 && argv[1][0] == 'v');
   poclbin = (argc > 1 && argv[1][0] == 'b');
   const char *explicit_binary_path = (argc > 2) ? argv[2] : NULL;
 
   const char *basename = "example0";
-  err = poclu_load_program (context, device, basename, spir, spirv, poclbin,
+  err = poclu_load_program (context, device, basename, spirv, poclbin,
                             explicit_binary_path, NULL, &program);
   if (err != CL_SUCCESS)
     goto FINISH;
@@ -125,6 +124,10 @@ FINISH:
   CHECK_CL_ERROR (clReleaseCommandQueue (queue));
   CHECK_CL_ERROR (clReleaseContext (context));
   CHECK_CL_ERROR (clUnloadPlatformCompiler (platform));
+  free(srcA);
+  free(srcB);
+  free(dst);
+
 
   return err;
 }
