@@ -45,9 +45,7 @@ struct pocl_context32 {
   uint work_dim;
 };
 
-/* The default pocl_context is 64b. It should be copied to a 32b one
-   when launching a kernel for a 32b target. */
-struct pocl_context {
+struct pocl_context64 {
   ulong num_groups[3];
   ulong global_offset[3];
   ulong local_size[3];
@@ -58,6 +56,19 @@ struct pocl_context {
   uint work_dim;
 };
 
+struct pocl_context {
+  size_t num_groups[3];
+  size_t global_offset[3];
+  size_t local_size[3];
+  uchar *printf_buffer;
+  uint *printf_buffer_position;
+  uint printf_buffer_capacity;
+  uchar *global_var_buffer;
+  uint work_dim;
+};
+
+
+
 /* Copy a 64b context struct to a 32b one. */
 #define POCL_CONTEXT_COPY64TO32(__DST, __SRC)				\
   do {									\
