@@ -402,17 +402,6 @@ do {
     }                                                                         \
   while (0)
 
-#define LL_COMPUTE_LENGTH(head, el, dst)                                      \
-  do                                                                          \
-    {                                                                         \
-      dst = 0;                                                                \
-      LL_FOREACH (head, el)                                                   \
-      {                                                                       \
-        ++dst;                                                                \
-      }                                                                       \
-    }                                                                         \
-  while (0)
-
 /******************************************************************************
  * doubly linked list macros (non-circular)                                   *
  *****************************************************************************/
@@ -545,4 +534,4 @@ do {
     }                                                                         \
   while (0)
 
-#endif /* UTLIST_H */
+#endif /* UTLIST_H */
\ No newline at end of file
