@@ -37,6 +37,9 @@ CL_API_SUFFIX__VERSION_1_0
 
   POCL_RETURN_ERROR_COND((command_queue->context == NULL), CL_INVALID_COMMAND_QUEUE);
 
+  POCL_RETURN_ERROR_COND ((*(command_queue->device->available) == CL_FALSE),
+                          CL_DEVICE_NOT_AVAILABLE);
+
   /* Even if we do not need to create a full command, the runtime requires it */
   pocl_create_command (&cmd, command_queue, CL_COMMAND_BARRIER, NULL, 0, NULL,
                        0, NULL, NULL);
