@@ -21,9 +21,9 @@
    IN THE SOFTWARE.
 */
 
-#include <assert.h>
-#include "pocl_util.h"
 #include "CL/cl_egl.h"
+#include "pocl_cl.h"
+#include "pocl_util.h"
 
 CL_API_ENTRY cl_int CL_API_CALL POname (clEnqueueReleaseEGLObjectsKHR) (
     cl_command_queue command_queue, cl_uint num_mem_objects,
