@@ -1,12 +1,13 @@
 /* OpenCL runtime library: clEnqueueSVMMap()
 
    Copyright (c) 2015 Michal Babej / Tampere University of Technology
+                 2023-2024 Pekka Jääskeläinen / Intel Finland Oy
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
 
    The above copyright notice and this permission notice shall be included in
@@ -16,9 +17,9 @@
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
 */
 
 #include "pocl_cl.h"
@@ -71,24 +72,23 @@ POname(clEnqueueSVMMap) (cl_command_queue command_queue,
         return CL_SUCCESS;
     }
 
-  _cl_command_node *cmd = NULL;
+  pocl_raw_ptr *svm_ptr_pocl
+      = pocl_find_raw_ptr_with_vm_ptr (context, svm_ptr);
 
-  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_SVM_MAP,
-                                 event, num_events_in_wait_list,
-                                 event_wait_list, 0, NULL, NULL);
-
-  if (errcode != CL_SUCCESS)
+  if (svm_ptr_pocl != NULL)
     {
-      POCL_MEM_FREE(cmd);
-      return errcode;
+      /* If it's nullptr, it must be a system allocation. */
+      assert (svm_ptr_pocl->shadow_cl_mem != NULL);
+      POname (clEnqueueMapBuffer (
+          command_queue, svm_ptr_pocl->shadow_cl_mem, CL_FALSE, 0 /* TODO */,
+          svm_ptr - svm_ptr_pocl->vm_ptr, size, num_events_in_wait_list,
+          event_wait_list, event, &errcode));
+      if (errcode != CL_SUCCESS)
+        return errcode;
+      if (event != NULL)
+        (*event)->command_type = CL_COMMAND_SVM_MAP;
     }
 
-  cmd->command.svm_map.svm_ptr = svm_ptr;
-  cmd->command.svm_map.size = size;
-  cmd->command.svm_map.flags = map_flags;
-
-  pocl_command_enqueue(command_queue, cmd);
-
   if (blocking_map == CL_TRUE)
     return POname(clFinish)(command_queue);
   else
