@@ -24,6 +24,7 @@
 
 #include "pocl_util.h"
 
+#ifdef DEVELOPER_MODE
 /* A version for querying the info and in case the device returns 
    a zero, assume the device info query hasn't been implemented 
    for the device driver at hand. Warns about an incomplete 
@@ -43,6 +44,16 @@
     POCL_WARN_INCOMPLETE ();                                                  \
   POCL_RETURN_GETINFO (__TYPE__, __VALUE__);
 
+#else
+
+#define POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(__TYPE__, __VALUE__)          \
+  POCL_RETURN_GETINFO (__TYPE__, __VALUE__);
+#define POCL_RETURN_DEVICE_INFO_WITH_IMG_CHECK(__TYPE__, __VALUE__)           \
+  POCL_RETURN_GETINFO (__TYPE__, __VALUE__);
+#define POCL_RETURN_DEVICE_INFO_WITH_EXT_CHECK(__TYPE__, __VALUE__, __EXT__)  \
+  POCL_RETURN_GETINFO (__TYPE__, __VALUE__);
+
+#endif
 
 CL_API_ENTRY cl_int CL_API_CALL
 POname(clGetDeviceInfo)(cl_device_id   device,
@@ -61,14 +72,14 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_IMAGE_SUPPORT:
     POCL_RETURN_GETINFO(cl_bool, device->image_support);
   case CL_DEVICE_TYPE:
-    POCL_RETURN_GETINFO(cl_device_type, device->type);   
+    POCL_RETURN_GETINFO (cl_device_type, device->type);
   case CL_DEVICE_VENDOR_ID:
     POCL_RETURN_GETINFO(cl_uint, device->vendor_id);
   case CL_DEVICE_MAX_COMPUTE_UNITS:
     POCL_RETURN_GETINFO(cl_uint, device->max_compute_units);
   case CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS          :
     POCL_RETURN_GETINFO(cl_uint, device->max_work_item_dimensions);
-  case CL_DEVICE_MAX_WORK_GROUP_SIZE               : 
+  case CL_DEVICE_MAX_WORK_GROUP_SIZE:
     {
       size_t max_wg_size = device->max_work_group_size;
       POCL_RETURN_GETINFO(size_t, max_wg_size);
@@ -319,10 +330,7 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_MAX_GLOBAL_VARIABLE_SIZE:
     POCL_RETURN_GETINFO(size_t, device->global_var_max_size);
   case CL_DEVICE_SPIR_VERSIONS:
-    if (device->supported_spir_versions)
-      POCL_RETURN_GETINFO_STR (device->supported_spir_versions);
-    else
-      POCL_RETURN_GETINFO_STR ("");
+    POCL_RETURN_GETINFO_STR ("");
   case CL_DEVICE_IL_VERSION:
     if (device->supported_spir_v_versions)
       POCL_RETURN_GETINFO_STR (device->supported_spir_v_versions);
@@ -345,7 +353,7 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_NON_UNIFORM_WORK_GROUP_SUPPORT:
     POCL_RETURN_GETINFO (cl_bool, device->non_uniform_work_group_support);
   case CL_DEVICE_WORK_GROUP_COLLECTIVE_FUNCTIONS_SUPPORT:
-    POCL_RETURN_GETINFO (cl_bool, CL_FALSE);
+    POCL_RETURN_GETINFO (cl_bool, device->wg_collective_func_support);
   case CL_DEVICE_GENERIC_ADDRESS_SPACE_SUPPORT:
     POCL_RETURN_GETINFO (cl_bool, device->generic_as_support);
   case CL_DEVICE_DEVICE_ENQUEUE_CAPABILITIES:
@@ -388,20 +396,72 @@ POname(clGetDeviceInfo)(cl_device_id   device,
 
   /** cl_khr_command_buffer queries **/
   case CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR:
-    POCL_RETURN_GETINFO (
-        cl_device_command_buffer_capabilities_khr,
-        CL_COMMAND_BUFFER_CAPABILITY_KERNEL_PRINTF_KHR
-            | CL_COMMAND_BUFFER_CAPABILITY_SIMULTANEOUS_USE_KHR
-            | CL_COMMAND_BUFFER_CAPABILITY_OUT_OF_ORDER_KHR);
+    POCL_RETURN_GETINFO (cl_device_command_buffer_capabilities_khr,
+                         CL_COMMAND_BUFFER_CAPABILITY_KERNEL_PRINTF_KHR
+                           | CL_COMMAND_BUFFER_CAPABILITY_SIMULTANEOUS_USE_KHR
+                           | CL_COMMAND_BUFFER_CAPABILITY_OUT_OF_ORDER_KHR
+                           | CL_COMMAND_BUFFER_CAPABILITY_MULTIPLE_QUEUE_KHR);
 
   case CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR:
     POCL_RETURN_GETINFO (cl_command_queue_properties, 0);
+
+  case CL_DEVICE_COMMAND_BUFFER_NUM_SYNC_DEVICES_KHR:
+    if (device->ops->get_device_info_ext != NULL
+        && device->ops->get_device_info_ext (device, param_name,
+                                             param_value_size, param_value,
+                                             param_value_size_ret)
+             == CL_SUCCESS)
+      return CL_SUCCESS;
+    /* If querying the actual device fails, it is probably safe to say there
+     * are no devices it can sync with on the device side */
+    POCL_RETURN_GETINFO (cl_uint, 0);
+
+  case CL_DEVICE_COMMAND_BUFFER_SYNC_DEVICES_KHR:
+    if (device->ops->get_device_info_ext != NULL)
+      /* Let devices fill in the list if supported, or no-op otherwise */
+      device->ops->get_device_info_ext (device, param_name, param_value_size,
+                                        param_value, param_value_size_ret);
+    return CL_SUCCESS; /* gracefully no-op in case the driver fails to handle
+                          the query */
+
+  /** cl_intel_unified_shared_memory queries **/
+  case CL_DEVICE_HOST_MEM_CAPABILITIES_INTEL:
+  case CL_DEVICE_DEVICE_MEM_CAPABILITIES_INTEL:
+  case CL_DEVICE_SINGLE_DEVICE_SHARED_MEM_CAPABILITIES_INTEL:
+  case CL_DEVICE_CROSS_DEVICE_SHARED_MEM_CAPABILITIES_INTEL:
+  case CL_DEVICE_SHARED_SYSTEM_MEM_CAPABILITIES_INTEL:
+    {
+      cl_bitfield caps;
+      switch (param_name)
+        {
+        case CL_DEVICE_HOST_MEM_CAPABILITIES_INTEL:
+          caps = device->host_usm_capabs;
+          break;
+        case CL_DEVICE_DEVICE_MEM_CAPABILITIES_INTEL:
+          caps = device->device_usm_capabs;
+          break;
+        case CL_DEVICE_SINGLE_DEVICE_SHARED_MEM_CAPABILITIES_INTEL:
+          caps = device->single_shared_usm_capabs;
+          break;
+        case CL_DEVICE_CROSS_DEVICE_SHARED_MEM_CAPABILITIES_INTEL:
+          caps = device->cross_shared_usm_capabs;
+          break;
+        case CL_DEVICE_SHARED_SYSTEM_MEM_CAPABILITIES_INTEL:
+          caps = device->system_shared_usm_capabs;
+          break;
+        default:
+          caps = 0;
+        }
+      POCL_RETURN_GETINFO (cl_bitfield, caps);
+    }
   }
 
-  if(device->ops->get_device_info_ext != NULL) {
-    return device->ops->get_device_info_ext(device, param_name, param_value_size,
-                                            param_value, param_value_size_ret);
-  }
+  if (device->ops->get_device_info_ext != NULL)
+    {
+      return device->ops->get_device_info_ext (device, param_name,
+                                               param_value_size, param_value,
+                                               param_value_size_ret);
+    }
 
   return CL_INVALID_VALUE;
 }
