@@ -45,6 +45,9 @@ POname(clGetSamplerInfo)(cl_sampler          sampler ,
       POCL_RETURN_GETINFO (cl_addressing_mode, sampler->addressing_mode);
     case CL_SAMPLER_FILTER_MODE:
       POCL_RETURN_GETINFO (cl_filter_mode, sampler->filter_mode);
+    case CL_SAMPLER_PROPERTIES:
+      POCL_RETURN_GETINFO_ARRAY (cl_sampler_properties,
+                                 sampler->num_properties, sampler->properties);
     }
 
   return CL_INVALID_VALUE;
