@@ -3,7 +3,7 @@
 
    Copyright (c) 2011-2013 Universidad Rey Juan Carlos
                  2011-2021 Pekka Jääskeläinen
-                 2022-2023 Pekka Jääskeläinen / Intel Finland Oy
+                 2022-2024 Pekka Jääskeläinen / Intel Finland Oy
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
@@ -593,7 +593,7 @@ pocl_exec_command (_cl_command_node *node)
     case CL_COMMAND_UNMAP_MEM_OBJECT:
       pocl_update_event_running (event);
       pocl_mem_identifier *mem_id = &mem->device_ptrs[dev->global_mem_id];
-      if (mem->is_image == CL_FALSE || IS_IMAGE1D_BUFFER (event->mem_objs[0]))
+      if (mem->is_image == CL_FALSE || IS_IMAGE1D_BUFFER (mem))
         {
           assert (dev->ops->unmap_mem != NULL);
           dev->ops->unmap_mem (dev->data, cmd->unmap.mem_id, mem,
@@ -641,12 +641,12 @@ pocl_exec_command (_cl_command_node *node)
           {
             void *ptr = cmd->svm_free.svm_pointers[i];
             POCL_LOCK_OBJ (event->context);
-            pocl_svm_ptr *tmp = NULL, *item = NULL;
-            DL_FOREACH_SAFE (event->context->svm_ptrs, item, tmp)
+            pocl_raw_ptr *tmp = NULL, *item = NULL;
+            DL_FOREACH_SAFE (event->context->raw_ptrs, item, tmp)
             {
-              if (item->svm_ptr == ptr)
+              if (item->vm_ptr == ptr)
                 {
-                  DL_DELETE (event->context->svm_ptrs, item);
+                  DL_DELETE (event->context->raw_ptrs, item);
                   break;
                 }
             }
@@ -724,6 +724,35 @@ pocl_exec_command (_cl_command_node *node)
       POCL_UPDATE_EVENT_COMPLETE_MSG (event, "Event SVM Mem_Advise        ");
       break;
 
+    case CL_COMMAND_SVM_MEMFILL_RECT_POCL:
+      pocl_update_event_running (event);
+      assert (dev->ops->svm_fill_rect);
+      dev->ops->svm_fill_rect (dev,
+                               cmd->svm_fill_rect.svm_ptr,
+                               cmd->svm_fill_rect.origin,
+                               cmd->svm_fill_rect.region,
+                               cmd->svm_fill_rect.row_pitch,
+                               cmd->svm_fill_rect.slice_pitch,
+                               cmd->svm_fill_rect.pattern,
+                               cmd->svm_fill_rect.pattern_size);
+      break;
+
+    case CL_COMMAND_SVM_MEMCPY_RECT_POCL:
+      pocl_update_event_running (event);
+      assert (dev->ops->svm_copy_rect);
+      dev->ops->svm_copy_rect (dev,
+                               cmd->svm_memcpy_rect.dst,
+                               cmd->svm_memcpy_rect.src,
+                               cmd->svm_memcpy_rect.dst_origin,
+                               cmd->svm_memcpy_rect.src_origin,
+                               cmd->svm_memcpy_rect.region,
+                               cmd->svm_memcpy_rect.dst_row_pitch,
+                               cmd->svm_memcpy_rect.dst_slice_pitch,
+                               cmd->svm_memcpy_rect.src_row_pitch,
+                               cmd->svm_memcpy_rect.src_slice_pitch);
+      POCL_UPDATE_EVENT_COMPLETE_MSG (event, "Event Copy Buffer Rect      ");
+      break;
+
     case CL_COMMAND_COMMAND_BUFFER_KHR:
       pocl_update_event_running (event);
       POCL_UPDATE_EVENT_COMPLETE (event);
@@ -735,6 +764,15 @@ pocl_exec_command (_cl_command_node *node)
     }
 }
 
+char *
+pocl_cpu_build_hash (cl_device_id device)
+{
+  char *res = calloc(1000, sizeof(char));
+  snprintf (res, 1000, "cpu-%s-%s", HOST_DEVICE_BUILD_HASH,
+            device->llvm_cpu);
+  return res;
+}
+
 /* call with brc_event UNLOCKED. */
 void
 pocl_broadcast (cl_event brc_event)
@@ -919,27 +957,24 @@ get_new_dlhandle_cache_item ()
 }
 
 void
-pocl_release_dlhandle_cache (_cl_command_node *cmd)
+pocl_release_dlhandle_cache (void *dlhandle_cache_item)
 {
-  pocl_dlhandle_cache_item *ci = NULL, *found = NULL;
+  pocl_dlhandle_cache_item *tmp = NULL, *ci = dlhandle_cache_item;
 
   POCL_LOCK (pocl_dlhandle_lock);
-  DL_FOREACH (pocl_dlhandle_cache, ci)
+  int found = CL_FALSE;
+  DL_FOREACH (pocl_dlhandle_cache, tmp)
   {
-    if ((memcmp (ci->hash, cmd->command.run.hash, sizeof (pocl_kernel_hash_t))
-         == 0)
-        && (ci->local_wgs[0] == cmd->command.run.pc.local_size[0])
-        && (ci->local_wgs[1] == cmd->command.run.pc.local_size[1])
-        && (ci->local_wgs[2] == cmd->command.run.pc.local_size[2]))
+    if (tmp == ci)
       {
-        found = ci;
+        found = CL_TRUE;
         break;
       }
   }
 
-  assert (found != NULL);
-  assert (found->ref_count > 0);
-  --found->ref_count;
+  assert (found == CL_TRUE);
+  assert (ci->ref_count > 0);
+  --ci->ref_count;
   POCL_UNLOCK (pocl_dlhandle_lock);
 }
 
@@ -1065,11 +1100,12 @@ fetch_dlhandle_cache_item (_cl_command_run *run_cmd, int specialize)
  * This can be useful in case we're just pre-compiling kernels
  * (or compiling them for binaries), and not actually need them immediately.
  *
- * TODO: This function is really specific to CPU (host) drivers since dlhandles
- * imply program loading to the same process as the host. Move to basic.c? */
-void
+ * Returns: a dlhandle cache item as void*; this needs to be given
+ * to pocl_release_dlhandle_cache(), if it was retained */
+void *
 pocl_check_kernel_dlhandle_cache (_cl_command_node *command,
-                                  int retain, int specialize)
+                                  int retain,
+                                  int specialize)
 {
   char workgroup_string[WORKGROUP_STRING_LENGTH];
   pocl_dlhandle_cache_item *ci = NULL;
@@ -1087,7 +1123,7 @@ pocl_check_kernel_dlhandle_cache (_cl_command_node *command,
     {
       if (retain) ++ci->ref_count;
       POCL_UNLOCK (pocl_dlhandle_lock);
-      return;
+      return ci;
     }
 
   /* Not found, build a new kernel and cache its dlhandle. */
@@ -1144,6 +1180,8 @@ pocl_check_kernel_dlhandle_cache (_cl_command_node *command,
 
   POCL_UNLOCK (pocl_dlhandle_lock);
   POCL_MEM_FREE (module_fn);
+
+  return ci;
 }
 
 #endif
@@ -1458,16 +1496,16 @@ static const cl_image_format supported_image_formats[] = {
 #endif
 
 void
-pocl_init_default_device_infos (cl_device_id dev)
+pocl_init_default_device_infos (cl_device_id dev,
+                                const char *device_extensions)
 {
   size_t i;
 
   dev->type = CL_DEVICE_TYPE_CPU;
   dev->max_work_item_dimensions = 3;
   dev->final_linkage_flags = final_ld_flags;
-  dev->extensions = DEFAULT_DEVICE_EXTENSIONS;
+  dev->extensions = device_extensions;
 
-  SETUP_DEVICE_CL_VERSION(HOST_DEVICE_CL_VERSION_MAJOR, HOST_DEVICE_CL_VERSION_MINOR)
   /*
     The hard restriction will be the context data which is
     stored in stack that can be as small as 8K in Linux.
@@ -1501,7 +1539,7 @@ pocl_init_default_device_infos (cl_device_id dev)
   dev->native_vector_width_long = POCL_DEVICES_NATIVE_VECTOR_WIDTH_LONG;
   dev->native_vector_width_float = POCL_DEVICES_NATIVE_VECTOR_WIDTH_FLOAT;
 
-  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_khr_fp64") == NULL)
+  if (strstr (device_extensions, "cl_khr_fp64") == NULL)
     {
       dev->native_vector_width_double = 0;
       dev->preferred_vector_width_double = 0;
@@ -1514,7 +1552,7 @@ pocl_init_default_device_infos (cl_device_id dev)
           = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_DOUBLE;
     }
 
-  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_khr_fp16") == NULL)
+  if (strstr (device_extensions, "cl_khr_fp16") == NULL)
     {
       dev->preferred_vector_width_half = 0;
       dev->native_vector_width_half = 0;
@@ -1567,7 +1605,7 @@ pocl_init_default_device_infos (cl_device_id dev)
 #endif
 #endif
 
-  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_khr_fp16") == NULL)
+  if (strstr (device_extensions, "cl_khr_fp16") == NULL)
     {
       dev->half_fp_config = 0;
     }
@@ -1576,7 +1614,7 @@ pocl_init_default_device_infos (cl_device_id dev)
       dev->half_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
     }
 
-  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_khr_fp64") == NULL)
+  if (strstr (device_extensions, "cl_khr_fp64") == NULL)
     {
       dev->double_fp_config = 0;
     }
@@ -1672,6 +1710,7 @@ pocl_init_default_device_infos (cl_device_id dev)
 #ifdef ENABLE_LLVM
 
   dev->llvm_target_triplet = OCL_KERNEL_TARGET;
+  dev->kernellib_fallback_name = NULL;
 
   char kernellib[POCL_MAX_PATHNAME_LENGTH] = "kernel-";
   char kernellib_fallback[POCL_MAX_PATHNAME_LENGTH];
@@ -1692,9 +1731,9 @@ pocl_init_default_device_infos (cl_device_id dev)
   strcpy(kernellib_fallback, kernellib);
   strcat(kernellib_fallback, OCL_KERNEL_TARGET_CPU);
   strcat(kernellib, dev->llvm_cpu);
+  dev->kernellib_fallback_name = strdup(kernellib_fallback);
 #endif
   dev->kernellib_name = strdup(kernellib);
-  dev->kernellib_fallback_name = strdup(kernellib_fallback);
   dev->kernellib_subdir = "host";
   dev->llvm_abi = pocl_get_llvm_cpu_abi ();
 
@@ -1708,7 +1747,7 @@ pocl_init_default_device_infos (cl_device_id dev)
 #endif
 
 #ifdef ENABLE_SPIRV
-  dev->supported_spir_v_versions = "SPIR-V_1.2";
+  dev->supported_spir_v_versions = "SPIR-V_1.2 SPIR-V_1.1 SPIR-V_1.0";
 #else
   dev->supported_spir_v_versions = "";
 #endif
@@ -1821,35 +1860,41 @@ pocl_setup_opencl_c_with_version (cl_device_id dev, int supports_30)
 /* this is a list of recognized extensions, not a list of reported extensions;
    the reported are stored in dev->extensions; this only for versioning */
 static const cl_name_version OPENCL_EXTENSIONS[]
-    = { { CL_MAKE_VERSION (1, 0, 0), "cl_intel_required_subgroup_size" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_intel_subgroups" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_intel_unified_shared_memory" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_byte_addressable_store" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_global_int32_base_atomics" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_global_int32_extended_atomics" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_local_int32_base_atomics" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_local_int32_extended_atomics" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_int64_base_atomics" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_int64_extended_atomics" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroups" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_extended_types" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_non_uniform_vote" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_ballot" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_non_uniform_arithmetic" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_shuffle" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_shuffle_relative" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_clustered_reduce" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_3d_image_writes" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_fp16" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_fp64" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_nv_device_attribute_query" },
-        { CL_MAKE_VERSION (2, 0, 0), "cl_khr_depth_images" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_khr_image2d_from_buffer" },
-        { CL_MAKE_VERSION (2, 1, 0), "cl_khr_spir" },
-        { CL_MAKE_VERSION (2, 1, 0), "cl_khr_il_program" },
-        { CL_MAKE_VERSION (0, 9, 4), "cl_khr_command_buffer" },
-        { CL_MAKE_VERSION (1, 0, 0), "cl_ext_float_atomics" },
-        { CL_MAKE_VERSION (0, 1, 0), "cl_pocl_pinned_buffers" }};
+  = { { CL_MAKE_VERSION (1, 0, 0), "cl_intel_required_subgroup_size" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_intel_subgroups" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_intel_subgroups_short" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_intel_unified_shared_memory" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_byte_addressable_store" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_global_int32_base_atomics" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_global_int32_extended_atomics" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_local_int32_base_atomics" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_local_int32_extended_atomics" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_int64_base_atomics" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_int64_extended_atomics" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroups" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_extended_types" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_non_uniform_vote" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_ballot" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_non_uniform_arithmetic" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_shuffle" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_shuffle_relative" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_subgroup_clustered_reduce" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_3d_image_writes" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_fp16" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_fp64" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_nv_device_attribute_query" },
+      { CL_MAKE_VERSION (2, 0, 0), "cl_khr_depth_images" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_khr_image2d_from_buffer" },
+      { CL_MAKE_VERSION (2, 1, 0), "cl_khr_il_program" },
+
+      { CL_MAKE_VERSION (0, 9, 4), "cl_khr_command_buffer" },
+      { CL_MAKE_VERSION (0, 9, 1), "cl_khr_command_buffer_multi_device" },
+      { CL_MAKE_VERSION (1, 0, 0), "cl_ext_float_atomics" },
+      { CL_MAKE_VERSION (0, 1, 0), "cl_ext_buffer_device_address" },
+      { CL_MAKE_VERSION (0, 9, 0), "cl_pocl_svm_rect" },
+      { CL_MAKE_VERSION (0, 9, 0), "cl_pocl_command_buffer_svm" },
+      { CL_MAKE_VERSION (0, 9, 0), "cl_pocl_command_buffer_host_buffer" },
+      { CL_MAKE_VERSION (0, 9, 0), "cl_pocl_command_buffer_host_exec" } };
 
 const size_t OPENCL_EXTENSIONS_NUM
     = sizeof (OPENCL_EXTENSIONS) / sizeof (OPENCL_EXTENSIONS[0]);
@@ -1940,7 +1985,7 @@ pocl_setup_ils_with_version (cl_device_id dev)
     }
 }
 
-static const cl_name_version OPENCL_FEATURES[] = {
+static const cl_name_version OPENCL_C_FEATURES[] = {
   { CL_MAKE_VERSION (3, 0, 0), "__opencl_c_3d_image_writes" },
   { CL_MAKE_VERSION (3, 0, 0), "__opencl_c_images" },
   { CL_MAKE_VERSION (3, 0, 0), "__opencl_c_read_write_images" },
@@ -1969,15 +2014,15 @@ static const cl_name_version OPENCL_FEATURES[] = {
   { CL_MAKE_VERSION (3, 0, 0), "__opencl_c_ext_fp64_local_atomic_min_max" },
 };
 
-const size_t OPENCL_FEATURES_NUM
-    = sizeof (OPENCL_FEATURES) / sizeof (OPENCL_FEATURES[0]);
+const size_t OPENCL_C_FEATURES_NUM
+    = sizeof (OPENCL_C_FEATURES) / sizeof (OPENCL_C_FEATURES[0]);
 
 void
 pocl_setup_features_with_version (cl_device_id dev)
 {
   cl_name_version *tmp = NULL;
   unsigned ret = pocl_space_delim_string_to_cl_name_version_array (
-      &tmp, dev->features, OPENCL_FEATURES, OPENCL_FEATURES_NUM);
+      &tmp, dev->features, OPENCL_C_FEATURES, OPENCL_C_FEATURES_NUM);
 
   dev->num_opencl_features_with_version = ret;
   dev->opencl_features_with_version = tmp;
