@@ -30,6 +30,7 @@
 #include <unistd.h>
 
 #include "pocl_cl.h"
+#include "pocl_timing.h"
 #include "utlist.h"
 
 // for pocl_aligned_malloc
@@ -136,21 +137,19 @@ pocl_driver_copy_with_size (void *data, pocl_mem_identifier *dst_mem_id,
     memcpy (dst_ptr + dst_offset, src_ptr + src_offset, size);
 }
 
+/* required for PoCL's command buffer extensions */
 void
-pocl_driver_copy_rect (void *data, pocl_mem_identifier *dst_mem_id,
-                       cl_mem dst_buf, pocl_mem_identifier *src_mem_id,
-                       cl_mem src_buf,
-                       const size_t *__restrict__ const dst_origin,
-                       const size_t *__restrict__ const src_origin,
-                       const size_t *__restrict__ const region,
-                       size_t const dst_row_pitch,
-                       size_t const dst_slice_pitch,
-                       size_t const src_row_pitch,
-                       size_t const src_slice_pitch)
+pocl_driver_svm_copy_rect (cl_device_id dev,
+                           void *__restrict__ dst_ptr,
+                           const void *__restrict__ src_ptr,
+                           const size_t *__restrict__ const dst_origin,
+                           const size_t *__restrict__ const src_origin,
+                           const size_t *__restrict__ const region,
+                           size_t dst_row_pitch,
+                           size_t dst_slice_pitch,
+                           size_t src_row_pitch,
+                           size_t src_slice_pitch)
 {
-
-  void *__restrict__ src_ptr = src_mem_id->mem_ptr;
-  void *__restrict__ dst_ptr = dst_mem_id->mem_ptr;
   char const *__restrict const adjusted_src_ptr
       = (char const *)src_ptr + src_origin[0] + src_row_pitch * src_origin[1]
         + src_slice_pitch * src_origin[2];
@@ -194,6 +193,74 @@ pocl_driver_copy_rect (void *data, pocl_mem_identifier *dst_mem_id,
     }
 }
 
+void
+pocl_driver_copy_rect (void *data,
+                       pocl_mem_identifier *dst_mem_id,
+                       cl_mem dst_buf,
+                       pocl_mem_identifier *src_mem_id,
+                       cl_mem src_buf,
+                       const size_t *__restrict__ const dst_origin,
+                       const size_t *__restrict__ const src_origin,
+                       const size_t *__restrict__ const region,
+                       size_t const dst_row_pitch,
+                       size_t const dst_slice_pitch,
+                       size_t const src_row_pitch,
+                       size_t const src_slice_pitch)
+{
+  void *__restrict__ src_ptr = src_mem_id->mem_ptr;
+  void *__restrict__ dst_ptr = dst_mem_id->mem_ptr;
+
+  pocl_driver_svm_copy_rect (NULL, dst_ptr, src_ptr, dst_origin, src_origin,
+                             region, dst_row_pitch, dst_slice_pitch,
+                             src_row_pitch, src_slice_pitch);
+}
+
+void
+pocl_driver_svm_fill_rect (cl_device_id dev,
+                           void *__restrict__ svm_ptr,
+                           const size_t *origin,
+                           const size_t *region,
+                           size_t row_pitch,
+                           size_t slice_pitch,
+                           void *__restrict__ pattern,
+                           size_t pattern_size)
+{
+  char *__restrict__ adjusted_ptr = (char *)svm_ptr + origin[0]
+                                    + row_pitch * origin[1]
+                                    + slice_pitch * origin[2];
+
+  POCL_MSG_PRINT_MEMORY ("FILL RECT \n"
+                         "PTR %p \n"
+                         "origin %u %u %u | region %u %u %u\n"
+                         "row_pitch %lu slice_pitch %lu\n",
+                         adjusted_ptr, (unsigned)origin[0],
+                         (unsigned)origin[1], (unsigned)origin[2],
+                         (unsigned)region[0], (unsigned)region[1],
+                         (unsigned)region[2], (unsigned long)row_pitch,
+                         (unsigned long)slice_pitch);
+
+  size_t j, k;
+
+  /* TODO: handle overlaping regions */
+  if ((row_pitch == region[0]) && (slice_pitch == (region[1] * region[0])))
+    {
+      size_t size = region[0] * region[1] * region[2];
+      pocl_fill_aligned_buf_with_pattern (adjusted_ptr, 0, size, pattern,
+                                          pattern_size);
+    }
+  else
+    {
+      for (k = 0; k < region[2]; ++k)
+        for (j = 0; j < region[1]; ++j)
+          {
+            size_t offset = row_pitch * j + slice_pitch * k;
+            size_t size = region[0];
+            pocl_fill_aligned_buf_with_pattern (adjusted_ptr, offset, size,
+                                                pattern, pattern_size);
+          }
+    }
+}
+
 void
 pocl_driver_write_rect (void *data, const void *__restrict__ const host_ptr,
                         pocl_mem_identifier *dst_mem_id, cl_mem dst_buf,
@@ -420,9 +487,9 @@ pocl_driver_alloc_mem_obj (cl_device_id device, cl_mem mem, void *host_ptr)
   p->mem_ptr = mem->mem_host_ptr;
   p->device_addr = p->mem_ptr;
 
-  /* If requesting device pinned memory, mark the mem such so it won't
-     get migrated away before being freed. */
-  if (mem->is_device_pinned)
+  /* If requesting memory with a fixed device address, make them pinned
+     so it won't get migrated away before being freed. */
+  if (mem->has_device_address)
     p->is_pinned = 1;
 
   POCL_MSG_PRINT_MEMORY ("Basic device ALLOC %p / size %zu \n", p->mem_ptr,
@@ -482,7 +549,7 @@ pocl_driver_svm_copy (cl_device_id dev,
 /* load LLVM IR binary from disk, deletes existing in-memory IR */
 static int
 pocl_reload_program_bc (char *program_bc_path, cl_program program,
-                      cl_uint device_i)
+                        cl_uint device_i)
 {
   char *temp_binary = NULL;
   uint64_t temp_size = 0;
@@ -491,7 +558,7 @@ pocl_reload_program_bc (char *program_bc_path, cl_program program,
     return -1;
   if (program->binaries[device_i])
     POCL_MEM_FREE (program->binaries[device_i]);
-  program->binaries[device_i] = temp_binary;
+  program->binaries[device_i] = (unsigned char*)temp_binary;
   program->binary_sizes[device_i] = temp_size;
   return 0;
 }
@@ -617,7 +684,7 @@ pocl_llvm_convert_and_link_ir (cl_program program, cl_uint device_i,
   if (program->program_il && program->program_il_size > 0)
     {
 #ifdef ENABLE_SPIRV
-      if (!strstr (device->extensions, "cl_khr_spir"))
+      if (!strstr (device->extensions, "cl_khr_il_program"))
         {
           APPEND_TO_BUILD_LOG_RET (CL_LINK_PROGRAM_FAILURE,
                                    "SPIR support is not available"
@@ -872,23 +939,6 @@ pocl_driver_supports_binary (cl_device_id device, size_t length,
     return 1;
 #endif
 
-#ifdef ENABLE_SPIR
-  /* SPIR binary is supported if the device has cl_khr_spir */
-  if (pocl_bitcode_is_triple (binary, length, "spir"))
-    {
-      if (strstr (device->extensions, "cl_khr_spir") == NULL)
-        {
-          POCL_MSG_WARN ("SPIR binary provided, but "
-                         "this device has no SPIR support\n");
-          return 0;
-        }
-      else
-        {
-          return 1;
-        }
-    }
-#endif
-
   /* LLVM IR can be supported by the driver, if the triple matches */
   if (device->llvm_target_triplet
       && pocl_bitcode_is_triple (binary, length, device->llvm_target_triplet))
@@ -1217,3 +1267,16 @@ pocl_cpu_gvar_init_callback(cl_program program, cl_uint dev_i,
 #endif
 }
 
+cl_int pocl_driver_get_synchronized_timestamps (cl_device_id dev,
+                                                cl_ulong *dev_timestamp,
+                                                cl_ulong *host_timestamp)
+{
+  uint64_t timestamp = pocl_gettimemono_ns();
+  if (dev_timestamp) {
+    *dev_timestamp = timestamp;
+  }
+  if (host_timestamp) {
+    *host_timestamp = timestamp;
+  }
+  return CL_SUCCESS;
+}
