@@ -32,7 +32,16 @@
 
 #include "pocl_cl.h"
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
 POCL_EXPORT
 void pocl_cpuinfo_detect_device_info(cl_device_id device);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* POCL_CPUINFO_H */
