@@ -1,6 +1,7 @@
 /* pocl-ptx-gen.cc - PTX code generation functions
 
     Copyright (c) 2016-2017 James Price / University of Bristol
+                  2024 Henry Linjamäki / Intel Finland Oy
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -40,14 +41,8 @@
 #include "llvm/IR/Module.h"
 #include "llvm/IR/Verifier.h"
 #include "llvm/Linker/Linker.h"
-#ifndef LLVM_OLDER_THAN_14_0
 #include "llvm/MC/TargetRegistry.h"
-#else
-#include "llvm/Support/TargetRegistry.h"
-#endif
-#ifndef LLVM_OLDER_THAN_11_0
 #include "llvm/Support/Alignment.h"
-#endif
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Target/TargetOptions.h"
 #include "llvm/Transforms/IPO.h"
@@ -103,7 +98,7 @@ static bool verifyModule(llvm::Module *Module, const char *step) {
 }
 
 int pocl_ptx_gen(void *llvm_module, const char *PTXFilename, const char *Arch,
-                 const char *LibDevicePath, int HasOffsets,
+                 unsigned PtxVersion, const char *LibDevicePath, int HasOffsets,
                  void **AlignmentMapPtr) {
 
   llvm::Module *Module = (llvm::Module *)llvm_module;
@@ -158,13 +153,8 @@ int pocl_ptx_gen(void *llvm_module, const char *PTXFilename, const char *Arch,
     POCL_MSG_PRINT_INFO("NVVM module:\n%s\n", ModuleString.c_str());
   }
 
-#ifdef LLVM_OLDER_THAN_11_0
-  llvm::StringRef Triple =
-      (sizeof(void *) == 8) ? "nvptx64-nvidia-cuda" : "nvptx-nvidia-cuda";
-#else
   std::string Triple =
       (sizeof(void *) == 8) ? "nvptx64-nvidia-cuda" : "nvptx-nvidia-cuda";
-#endif
 
   std::string Error;
   // Get NVPTX target.
@@ -180,13 +170,13 @@ int pocl_ptx_gen(void *llvm_module, const char *PTXFilename, const char *Arch,
   // TODO: Set options?
   llvm::TargetOptions Options;
 
-  // TODO: CPU and features?
+  auto Features = std::string("+ptx") + std::to_string(PtxVersion);
 #ifdef LLVM_OLDER_THAN_16_0
   std::unique_ptr<llvm::TargetMachine> Machine(
-      Target->createTargetMachine(Triple, Arch, "+ptx40", Options, llvm::None));
+      Target->createTargetMachine(Triple, Arch, Features, Options, llvm::None));
 #else
-  std::unique_ptr<llvm::TargetMachine> Machine(
-      Target->createTargetMachine(Triple, Arch, "+ptx40", Options, std::nullopt));
+  std::unique_ptr<llvm::TargetMachine> Machine(Target->createTargetMachine(
+      Triple, Arch, Features, Options, std::nullopt));
 #endif
   llvm::legacy::PassManager Passes;
 
@@ -195,7 +185,13 @@ int pocl_ptx_gen(void *llvm_module, const char *PTXFilename, const char *Arch,
   llvm::raw_svector_ostream PTXStream(Data);
   if (Machine->addPassesToEmitFile(Passes, PTXStream,
                                    nullptr,
-                                   llvm::CGFT_AssemblyFile)) {
+#if LLVM_MAJOR < 18
+                                   llvm::CGFT_AssemblyFile))
+#else
+                                   llvm::CodeGenFileType::AssemblyFile))
+#endif
+  {
+
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to add passes\n");
     return CL_BUILD_PROGRAM_FAILURE;
   }
@@ -203,17 +199,11 @@ int pocl_ptx_gen(void *llvm_module, const char *PTXFilename, const char *Arch,
   // Run passes.
   Passes.run(*Module);
 
-#ifdef LLVM_OLDER_THAN_11_0
-  std::string PTX = PTXStream.str();
-  const char *Content = PTX.c_str();
-  size_t ContentSize = PTX.size();
-#else
   llvm::StringRef PTX = PTXStream.str();
   const char *Content = PTX.data();
   size_t ContentSize = PTX.size();
-#endif
 
-  if (pocl_write_file(PTXFilename, Content, ContentSize, 0, 0)) {
+  if (pocl_write_file(PTXFilename, Content, ContentSize, 0)) {
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to write final PTX into %s\n",
                  PTXFilename);
     return CL_BUILD_PROGRAM_FAILURE;
@@ -295,25 +285,13 @@ void fixPrintF(llvm::Module *Module) {
 #endif
 
   // Create i32 to hold current argument index.
-#ifdef LLVM_OLDER_THAN_11_0
-  llvm::AllocaInst *ArgIndexPtr =
-      new llvm::AllocaInst(I32, 0, llvm::ConstantInt::get(I32, 1));
-  ArgIndexPtr->insertBefore(&*NewPrintF->begin()->begin());
-#else
   llvm::AllocaInst *ArgIndexPtr =
       new llvm::AllocaInst(I32, 0, llvm::ConstantInt::get(I32, 1), llvm::Align(4));
   ArgIndexPtr->insertBefore(&*NewPrintF->begin()->begin());
-#endif
 
-#ifdef LLVM_OLDER_THAN_11_0
-  llvm::StoreInst *ArgIndexInit =
-      new llvm::StoreInst(llvm::ConstantInt::get(I32, 0), ArgIndexPtr);
-  ArgIndexInit->insertAfter(ArgIndexPtr);
-#else
   llvm::StoreInst *ArgIndexInit =
       new llvm::StoreInst(llvm::ConstantInt::get(I32, 0), ArgIndexPtr, false, llvm::Align(4));
   ArgIndexInit->insertAfter(ArgIndexPtr);
-#endif
 
   // Replace calls to _cl_va_arg with reads from new i64 array argument.
   llvm::Function *VaArgFunc = Module->getFunction("__cl_va_arg");
@@ -329,13 +307,8 @@ void fixPrintF(llvm::Module *Module) {
         continue;
 
       // Get current argument index.
-#ifdef LLVM_OLDER_THAN_11_0
-      llvm::LoadInst *ArgIndex = new llvm::LoadInst(ArgIndexPtr);
-      ArgIndex->insertBefore(Call);
-#else
       llvm::LoadInst *ArgIndex = new llvm::LoadInst(I32, ArgIndexPtr, "poclCudaLoad", false, llvm::Align(4));
       ArgIndex->insertBefore(Call);
-#endif
       // Get pointer to argument data.
       llvm::Value *ArgOut = Call->getArgOperand(1);
       llvm::GetElementPtrInst *ArgIn =
@@ -348,29 +321,19 @@ void fixPrintF(llvm::Module *Module) {
       ArgOut = ArgOutBC;
 
       // Load argument.
-#ifdef LLVM_OLDER_THAN_11_0
-      llvm::LoadInst *ArgValue = new llvm::LoadInst(ArgIn);
-      ArgValue->insertAfter(ArgIn);
-      llvm::StoreInst *ArgStore = new llvm::StoreInst(ArgValue, ArgOut);
-      ArgStore->insertAfter(ArgOutBC);
-#else
       llvm::LoadInst *ArgValue = new llvm::LoadInst(I64, ArgIn, "poclCudaArgLoad", false, llvm::Align(8));
       ArgValue->insertAfter(ArgIn);
       llvm::StoreInst *ArgStore = new llvm::StoreInst(ArgValue, ArgOut, false, llvm::Align(8));
       ArgStore->insertAfter(ArgOutBC);
-#endif
+
       // Increment argument index.
       llvm::BinaryOperator *Inc = llvm::BinaryOperator::Create(
           llvm::BinaryOperator::Add, ArgIndex, llvm::ConstantInt::get(I32, 1));
       Inc->insertAfter(ArgIndex);
 
-#ifdef LLVM_OLDER_THAN_11_0
-      llvm::StoreInst *StoreInc = new llvm::StoreInst(Inc, ArgIndexPtr);
-      StoreInc->insertAfter(Inc);
-#else
       llvm::StoreInst *StoreInc = new llvm::StoreInst(Inc, ArgIndexPtr, false, llvm::Align(4));
       StoreInc->insertAfter(Inc);
-#endif
+
       // Remove call to _cl_va_arg.
       Call->eraseFromParent();
     }
@@ -394,15 +357,9 @@ void fixPrintF(llvm::Module *Module) {
 
     // Allocate array for arguments.
     // TODO: Deal with vector arguments.
-#ifdef LLVM_OLDER_THAN_11_0
-    llvm::AllocaInst *Args =
-        new llvm::AllocaInst(I64, 0, llvm::ConstantInt::get(I32, NumArgs));
-    Args->insertBefore(Call);
-#else
     llvm::AllocaInst *Args =
         new llvm::AllocaInst(I64, 0, llvm::ConstantInt::get(I32, NumArgs), llvm::Align(8));
     Args->insertBefore(Call);
-#endif
 
     // Loop over arguments (skipping format).
     for (unsigned A = 0; A < NumArgs; A++) {
@@ -440,13 +397,8 @@ void fixPrintF(llvm::Module *Module) {
       }
 #endif
       // Store argument to i64 array.
-#ifdef LLVM_OLDER_THAN_11_0
-      llvm::StoreInst *Store = new llvm::StoreInst(Arg, ArgPtr);
-      Store->insertBefore(Call);
-#else
       llvm::StoreInst *Store = new llvm::StoreInst(Arg, ArgPtr, false, llvm::Align(8));
       Store->insertBefore(Call);
-#endif
     }
 
     // Fix address space of undef format values.
@@ -687,8 +639,9 @@ int linkLibDevice(llvm::Module *Module, const char *LibDevicePath) {
   // Load libdevice bitcode library.
   llvm::Expected<std::unique_ptr<llvm::Module>> LibDeviceModule =
       parseBitcodeFile(Buffer->get()->getMemBufferRef(), Module->getContext());
-  if (!LibDeviceModule) {
-    POCL_MSG_ERR("[CUDA] failed to load libdevice bitcode\n");
+  if (auto Error = LibDeviceModule.takeError()) {
+    POCL_MSG_ERR("[CUDA] failed to load libdevice bitcode:\n%s\n",
+                 toString(std::move(Error)).c_str());
     return -1;
   }
 
