@@ -22,6 +22,7 @@
    THE SOFTWARE.
 */
 
+
 #define _GNU_SOURCE
 
 #include <string.h>
@@ -53,6 +54,8 @@
 #include "pocl_export.h"
 #include "pocl_version.h"
 
+#include "utlist_addon.h"
+
 #ifdef ENABLE_RDMA
 #include "pocl_rdma.h"
 #endif
@@ -67,6 +70,9 @@
 #ifdef BUILD_PTHREAD
 #include "pthread/pocl-pthread.h"
 #endif
+#ifdef BUILD_TBB
+#include "tbb/tbb.h"
+#endif
 
 #ifdef TCE_AVAILABLE
 #include "tce/ttasim/ttasim.h"
@@ -100,9 +106,6 @@
 
 #ifdef BUILD_REMOTE_CLIENT
 #include "remote/remote.h"
-extern cl_int pocl_remote_setup_peer_mesh ();
-// from remote/remote.c
-extern const char *remote_device_name_ptr;
 #endif
 
 #define MAX_DEV_NAME_LEN 64
@@ -120,6 +123,14 @@ extern const char *remote_device_name_ptr;
 
 
 /* the enabled devices */
+/*
+  IMPORTANT: utlist_addon.h macros are used to atomically access
+  pocl_devices' next pointers. pocl_devices should only be accessed
+  through these macros. LL_DELETE shouldn't be used as a devices' address
+  must remain valid. LL_PREPEND shouldn't be used as well as it modifies the
+  head.
+*/
+/* Head for the pocl_devices linked list*/
 struct _cl_device_id *pocl_devices = NULL;
 unsigned int pocl_num_devices = 0;
 
@@ -132,10 +143,19 @@ unsigned int pocl_num_devices = 0;
 const char *
 pocl_get_device_name (unsigned index)
 {
-  if (index < pocl_num_devices)
-    return pocl_devices[index].long_name;
-  else
-    return NULL;
+
+  if (index < POCL_ATOMIC_LOAD (pocl_num_devices))
+    {
+      cl_device_id device;
+      unsigned i = 0;
+      LL_FOREACH_ATOMIC (pocl_devices, device)
+      {
+        if (i == index)
+          return device->long_name;
+        i++;
+        }
+    }
+  return NULL;
 }
 
 /* Init function prototype */
@@ -152,6 +172,9 @@ static init_device_ops pocl_devices_init_ops[] = {
 #ifdef TCE_AVAILABLE
   INIT_DEV (ttasim),
 #endif
+#ifdef BUILD_TBB
+  INIT_DEV (tbb),
+#endif
 #ifdef BUILD_HSA
   INIT_DEV (hsa),
 #endif
@@ -177,13 +200,16 @@ static init_device_ops pocl_devices_init_ops[] = {
 
 #define POCL_NUM_DEVICE_TYPES (sizeof(pocl_devices_init_ops) / sizeof((pocl_devices_init_ops)[0]))
 
-char pocl_device_types[POCL_NUM_DEVICE_TYPES][30] = {
+char pocl_device_types[POCL_NUM_DEVICE_TYPES][33] = {
 #ifdef BUILD_BASIC
   "basic",
 #endif
 #ifdef BUILD_PTHREAD
   "pthread",
 #endif
+#ifdef BUILD_TBB
+  "tbb",
+#endif
 #ifdef TCE_AVAILABLE
   "ttasim",
 #endif
@@ -212,7 +238,6 @@ char pocl_device_types[POCL_NUM_DEVICE_TYPES][30] = {
 
 static struct pocl_device_ops pocl_device_ops[POCL_NUM_DEVICE_TYPES];
 
-extern pocl_lock_t pocl_runtime_config_lock;
 extern pocl_lock_t pocl_context_handling_lock;
 
 POCL_EXPORT int pocl_offline_compile = 0;
@@ -311,7 +336,8 @@ unsigned int
 pocl_get_devices (cl_device_type device_type, cl_device_id *devices,
                   unsigned int num_devices)
 {
-  unsigned int i, dev_added = 0;
+  unsigned int dev_added = 0;
+  cl_device_id device;
 
   cl_device_type device_type_tmp = device_type;
   if (device_type_tmp == CL_DEVICE_TYPE_ALL)
@@ -319,30 +345,30 @@ pocl_get_devices (cl_device_type device_type, cl_device_id *devices,
       device_type_tmp = ~CL_DEVICE_TYPE_CUSTOM;
     }
 
-  for (i = 0; i < pocl_num_devices; ++i)
-    {
-      if (!pocl_offline_compile && (*pocl_devices[i].available == CL_FALSE))
-        continue;
+  LL_FOREACH_ATOMIC (pocl_devices, device)
+  {
+    if (!pocl_offline_compile && (*device->available == CL_FALSE))
+      continue;
 
-      if (device_type_tmp == CL_DEVICE_TYPE_DEFAULT)
-        {
-          devices[dev_added] = &pocl_devices[i];
-          ++dev_added;
-          break;
-        }
+    if (device_type_tmp == CL_DEVICE_TYPE_DEFAULT)
+      {
+        devices[dev_added] = device;
+        ++dev_added;
+        break;
+      }
 
-      if (pocl_devices[i].type & device_type_tmp)
-        {
-            if (dev_added < num_devices)
-              {
-                devices[dev_added] = &pocl_devices[i];
-                ++dev_added;
-              }
-            else
-              {
-                break;
-              }
-        }
+    if (device->type & device_type_tmp)
+      {
+        if (dev_added < num_devices)
+          {
+            devices[dev_added] = device;
+            ++dev_added;
+          }
+        else
+          {
+            break;
+          }
+      }
     }
   return dev_added;
 }
@@ -351,7 +377,7 @@ unsigned int
 pocl_get_device_type_count(cl_device_type device_type)
 {
   unsigned int count = 0;
-  unsigned int i;
+  cl_device_id device;
 
   cl_device_type device_type_tmp = device_type;
   if (device_type_tmp == CL_DEVICE_TYPE_ALL)
@@ -359,18 +385,18 @@ pocl_get_device_type_count(cl_device_type device_type)
       device_type_tmp = ~CL_DEVICE_TYPE_CUSTOM;
     }
 
-  for (i = 0; i < pocl_num_devices; ++i)
-    {
-      if (!pocl_offline_compile && (*pocl_devices[i].available == CL_FALSE))
-        continue;
+  LL_FOREACH_ATOMIC (pocl_devices, device)
+  {
+    if (!pocl_offline_compile && (*device->available == CL_FALSE))
+      continue;
 
-      if (device_type_tmp == CL_DEVICE_TYPE_DEFAULT)
-        return 1;
+    if (device_type_tmp == CL_DEVICE_TYPE_DEFAULT)
+      return 1;
 
-      if (pocl_devices[i].type & device_type_tmp)
-        {
-           ++count;
-        }
+    if (device->type & device_type_tmp)
+      {
+        ++count;
+      }
     }
 
   return count;
@@ -383,40 +409,42 @@ pocl_uninit_devices ()
   cl_int retval = CL_SUCCESS;
 
   POCL_LOCK (pocl_init_lock);
-  if ((!devices_active) || (pocl_num_devices == 0))
+  if ((!devices_active) || (POCL_ATOMIC_LOAD (pocl_num_devices) == 0))
     goto FINISH;
 
   POCL_MSG_PRINT_GENERAL ("UNINIT all devices\n");
 
-  unsigned i, j, dev_index;
+  unsigned i, j;
+  cl_device_id device = pocl_devices;
 
-  dev_index = 0;
   cl_device_id d;
   for (i = 0; i < POCL_NUM_DEVICE_TYPES; ++i)
     {
       if (pocl_devices_init_ops[i] == NULL)
         continue;
       assert (pocl_device_ops[i].init);
-      for (j = 0; j < device_count[i]; ++j)
-        {
-          d = &pocl_devices[dev_index];
-          if (*(d->available) == CL_FALSE)
-              continue;
-          if (d->ops->reinit == NULL || d->ops->uninit == NULL)
-            continue;
-          cl_int ret = d->ops->uninit (j, d);
-          if (ret != CL_SUCCESS)
-            {
-              retval = ret;
-              goto FINISH;
-            }
+
+      j = 0;
+      LL_FOREACH_ATOMIC (device, device)
+      {
+        d = device;
+        if (*(d->available) == CL_FALSE)
+          continue;
+        if (d->ops->reinit == NULL || d->ops->uninit == NULL)
+          continue;
+        cl_int ret = d->ops->uninit (j, d);
+        if (ret != CL_SUCCESS)
+          {
+            retval = ret;
+            goto FINISH;
+          }
 #ifdef ENABLE_LOADABLE_DRIVERS
           if (pocl_device_handles[i] != NULL)
             {
               dlclose (pocl_device_handles[i]);
             }
 #endif
-          ++dev_index;
+          j++;
         }
     }
 
@@ -436,14 +464,14 @@ pocl_reinit_devices ()
   if (devices_active)
     return retval;
 
-  if (pocl_num_devices == 0)
+  if (POCL_ATOMIC_LOAD (pocl_num_devices) == 0)
     return CL_DEVICE_NOT_FOUND;
 
   POCL_MSG_WARN ("REINIT all devices\n");
 
-  unsigned i, j, dev_index;
+  unsigned i, j;
+  cl_device_id device = pocl_devices;
 
-  dev_index = 0;
   char env_name[1024];
   char dev_name[MAX_DEV_NAME_LEN] = { 0 };
   cl_device_id d;
@@ -452,22 +480,24 @@ pocl_reinit_devices ()
     {
       pocl_str_toupper (dev_name, pocl_device_ops[i].device_name);
       assert (pocl_device_ops[i].init);
-      for (j = 0; j < device_count[i]; ++j)
-        {
-          d = &pocl_devices[dev_index];
-          if (*(d->available) == CL_FALSE)
-            continue;
-          if (d->ops->reinit == NULL || d->ops->uninit == NULL)
-            continue;
-          snprintf (env_name, 1024, "POCL_%s%d_PARAMETERS", dev_name, j);
-          cl_int ret = d->ops->reinit (j, d, getenv (env_name));
-          if (ret != CL_SUCCESS)
-            {
-              retval = ret;
-              goto FINISH;
-            }
 
-          ++dev_index;
+      j = 0;
+      LL_FOREACH_ATOMIC (device, device)
+      {
+        d = device;
+        if (*(d->available) == CL_FALSE)
+          continue;
+        if (d->ops->reinit == NULL || d->ops->uninit == NULL)
+          continue;
+        snprintf (env_name, 1024, "POCL_%s%d_PARAMETERS", dev_name, j);
+        cl_int ret = d->ops->reinit (j, d, getenv (env_name));
+        if (ret != CL_SUCCESS)
+          {
+            retval = ret;
+            goto FINISH;
+          }
+
+        j++;
         }
     }
 
@@ -506,7 +536,6 @@ pocl_init_devices ()
     }
   else
     {
-      POCL_INIT_LOCK (pocl_runtime_config_lock);
       POCL_INIT_LOCK (pocl_context_handling_lock);
     }
 
@@ -534,8 +563,7 @@ pocl_init_devices ()
     sleep (delay);
 #endif
 
-
-#ifdef __linux__
+#if defined(__linux__) && !defined(__ANDROID__)
 
 #ifdef ENABLE_HOST_CPU_DEVICES
   if (pocl_get_bool_option ("POCL_SIGFPE_HANDLER", 1))
@@ -619,10 +647,6 @@ pocl_init_devices ()
   POCL_GOTO_ERROR_ON ((pocl_num_devices == 0), CL_DEVICE_NOT_FOUND,
                       "no devices found. %s=%s\n", POCL_DEVICES_ENV, dev_env);
 
-  pocl_devices = (struct _cl_device_id*) calloc(pocl_num_devices, sizeof(struct _cl_device_id));
-  POCL_GOTO_ERROR_ON ((pocl_devices == NULL), CL_OUT_OF_HOST_MEMORY,
-                      "Can not allocate memory for devices\n");
-
   dev_index = 0;
   /* Init infos for each probed devices */
   for (i = 0; i < POCL_NUM_DEVICE_TYPES; ++i)
@@ -634,7 +658,9 @@ pocl_init_devices ()
 
       for (j = 0; j < device_count[i]; ++j)
         {
-          cl_device_id dev = &pocl_devices[dev_index];
+          cl_device_id dev;
+          dev = (cl_device_id)calloc (1, sizeof (*dev));
+
           dev->ops = &pocl_device_ops[i];
           dev->dev_id = dev_index;
           /* The default value for the global memory space identifier is
@@ -643,7 +669,9 @@ pocl_init_devices ()
              a shared global memory. */
           dev->global_mem_id = dev_index;
           POCL_INIT_OBJECT (dev);
-          dev->driver_version = POCL_VERSION_FULL;
+          dev->driver_version = pocl_get_string_option (
+              "POCL_DRIVER_VERSION_OVERRIDE", POCL_VERSION_FULL);
+
           if (dev->version == NULL)
             dev->version = "OpenCL 2.0 pocl";
 
@@ -655,20 +683,20 @@ pocl_init_devices ()
                < 0),
               CL_OUT_OF_HOST_MEMORY, "Unable to generate the env string.");
 
-          errcode = pocl_devices[dev_index].ops->init (
-              j, &pocl_devices[dev_index], getenv (env_name));
+          errcode = dev->ops->init (j, dev, getenv (env_name));
           POCL_GOTO_ERROR_ON ((errcode != CL_SUCCESS), errcode,
                               "Device %i / %s initialization failed!", j,
                               dev_name);
 
+          LL_APPEND_ATOMIC (pocl_devices, dev);
+
           ++dev_index;
         }
+      if (pocl_device_ops[i].post_init != NULL)
+        {
+          pocl_device_ops[i].post_init(&pocl_device_ops[i]);
+        }
     }
-
-#ifdef BUILD_REMOTE_CLIENT
-  pocl_remote_setup_peer_mesh ();
-#endif
-
   first_init_done = 1;
   devices_active = 1;
 ERROR:
