@@ -28,6 +28,10 @@
 #define POCL_REMOTE_COMMUNICATION_H
 
 #include "messages.h"
+#include "pocl.h"
+
+#include "utlist_addon.h"
+#include "utlist.h"
 
 #ifdef ENABLE_RDMA
 #include "pocl_rdma.h"
@@ -65,6 +69,8 @@ typedef void (*network_command_callback) (void *arg, _cl_command_node *node,
 
 typedef struct network_command network_command;
 
+typedef struct remote_device_data_s remote_device_data_t;
+
 /** Lock used for synchronous commands */
 typedef struct sync_t
 {
@@ -91,6 +97,7 @@ typedef struct async_t
 #define CREATE_ASYNC_NETCMD                                                   \
   network_command *netcmd = calloc (1, sizeof (network_command));             \
   netcmd->status = NETCMD_STARTED;                                            \
+  POCL_LOCK_OBJ (node->sync.event.event);                                     \
   netcmd->event_id = node->sync.event.event->id;                              \
   struct event_node *n;                                                       \
   LL_COMPUTE_LENGTH (node->sync.event.event->wait_list, n,                    \
@@ -105,6 +112,7 @@ typedef struct async_t
       *(dst++) = n->event->id;                                                \
     }                                                                         \
   }                                                                           \
+  POCL_UNLOCK_OBJ (node->sync.event.event);                                   \
   netcmd->receiver = data->inflight_queue;                                    \
   netcmd->synchronous = 0;                                                    \
   netcmd->data.async.cb = cb;                                                 \
@@ -180,7 +188,8 @@ typedef struct remote_server_data_s
   struct remote_server_data_s *next;
   struct remote_server_data_s *prev;
 
-  uint8_t session[SESSION_ID_LENGTH];
+  uint64_t session;
+  uint8_t authkey[AUTHKEY_LENGTH];
   uint32_t available;
   sync_t setup_lock;
   int threads_awaiting_reconnect;
@@ -252,7 +261,8 @@ typedef struct remote_server_data_s
     }                                                                         \
   while (0)
 
-#define UNSET_REMOTE_ID(map, id)                                              \
+
+#define UNSET_REMOTE_ID(map, id)                                        \
   do                                                                          \
     {                                                                         \
       small_vector_remove_##map##_ids (data, id);                             \
@@ -270,54 +280,6 @@ typedef struct remote_queue_data_s
   char *printf_buffer;
 } remote_queue_data_t;
 
-typedef struct remote_device_data_s
-{
-  remote_server_data_t *server;
-  cl_image_format *supported_image_formats;
-  char *build_hash;
-  unsigned remote_device_index;
-  unsigned remote_platform_index;
-  unsigned local_did;
-
-  /* SVM support: The SVM memory pool regions in the remote device's
-     and host's memories. */
-  size_t device_svm_region_start_addr;
-  size_t device_svm_region_size;
-
-  /* A context to store device-wide data (currently only the pinned memory
-     buffer for the SVM allocations). */
-  cl_context device_context;
-  cl_mem pinned_device_allocation;
-
-  size_t host_svm_region_start_addr;
-  size_t host_svm_region_size;
-
-  /* The difference between host and device SVM region starting
-     addresses (device minus host start address). That is, this offset
-     must be added to host SVM addresses to end up with a device SVM
-     address and vice versa. The addition can wraparound, which is defined
-     behavior with unsigned values in C. Ideally, this offset would
-     be always zero to avoid address translation overheads, but
-     it's difficult to guarantee, so generally we must be ready for
-     a non-zero offset and deal with it. */
-  size_t svm_region_offset;
-
-  // migrated -> ready to launch queue
-  _cl_command_node *work_queue;
-  // finished queue
-  _cl_command_node *finished_list;
-
-  // driver wake + lock
-  ALIGN_CACHE (pocl_lock_t wq_lock);
-  ALIGN_CACHE (pocl_cond_t wakeup_cond);
-  ALIGN_CACHE (pocl_lock_t mem_lock);
-
-  /* device pthread */
-  pocl_thread_t driver_thread_id;
-  size_t driver_thread_exit_requested;
-
-} remote_device_data_t;
-
 typedef struct kernel_data_s
 {
   char *pod_arg_storage;
@@ -351,8 +313,7 @@ cl_int pocl_network_setup_devinfo (cl_device_id device,
                                    remote_server_data_t *data, uint32_t pid,
                                    uint32_t did);
 
-cl_int pocl_network_create_buffer (remote_device_data_t *d, uint32_t mem_id,
-                                   uint32_t mem_flags, uint64_t mem_size,
+cl_int pocl_network_create_buffer (remote_device_data_t *d, cl_mem mem,
                                    void **device_addr);
 
 cl_int pocl_network_free_buffer (remote_device_data_t *d, uint64_t mem_id,
@@ -370,12 +331,13 @@ cl_int pocl_network_setup_metadata (char *buffer, size_t total_size,
                                     cl_program program, size_t *num_kernels,
                                     pocl_kernel_metadata_t **kernel_meta);
 
-cl_int pocl_network_build_program (
+cl_int pocl_network_build_or_link_program (
     remote_device_data_t *ddata, const void *payload, size_t payload_size,
     int is_binary, int is_builtin, int is_spirv, uint32_t prog_id,
     const char *options, char **kernel_meta_bytes, size_t *kernel_meta_size,
     uint32_t *devices, uint32_t *platforms, size_t num_devices,
-    char **build_log, char **binaries, size_t *binary_sizes);
+    char **build_log, char **binaries, size_t *binary_sizes,
+    size_t svm_region_offset, int compile_only, int link_only);
 
 cl_int pocl_network_free_program (remote_device_data_t *ddata,
                                   uint32_t prog_id);
