@@ -123,18 +123,26 @@ class TCEDevice {
   uint32_t statusAddr;
 
   uint32_t curKernelAddr;
+  cl_bool available;
   cl_kernel curKernel;
 
   size_t curLocalX;
   size_t curLocalY;
   size_t curLocalZ;
 
+  size_t curGoffsX;
+  size_t curGoffsY;
+  size_t curGoffsZ;
+
   uint64_t globalCycleCount;
 
   pocl_lock_t wq_lock;
   pocl_cond_t wakeup_cond;
   pocl_lock_t tce_compile_lock;
   _cl_command_node *work_queue;
+
+  chunk_info_t *printf_buffer;
+  chunk_info_t *printf_position_chunk;
 };
 
 void *pocl_tce_driver_thread (void *cldev);
