@@ -249,13 +249,12 @@ process_options (const char *options, char *modded_options, char *link_options,
               /* the LLVM API call pushes the parameters directly to the
                  frontend without using -Xclang */
 
-#ifndef LLVM_OLDER_THAN_11_0
             // LLVM 11 has removed "-cl-denorms-are-zero" option
             // https://reviews.llvm.org/D69878
             if (strncmp(token, "-cl-denorms-are-zero", 20) == 0) {
                 token = "-fdenormal-fp-math=positive-zero";
             }
-#endif
+
             if (strncmp (token, "-cl-std=CL", 10) == 0)
             {
                 unsigned major = token[10] - '0';
@@ -280,13 +279,7 @@ process_options (const char *options, char *modded_options, char *link_options,
         }
       else if (strncmp (token, "-g", 2) == 0)
         {
-#ifdef LLVM_OLDER_THAN_11_0
-          token = "-dwarf-column-info -debug-info-kind=limited " \
-	    "-dwarf-version=4 -debugger-tuning=gdb";
-#else
-          token = "-debug-info-kind=limited " \
-	    "-dwarf-version=4 -debugger-tuning=gdb";
-#endif
+          APPEND_TOKEN ();
         }
       else if (strncmp (token, "-D", 2) == 0 || strncmp (token, "-I", 2) == 0)
         {
@@ -729,7 +722,7 @@ compile_and_link_program(int compile_program,
     }
   else
     {
-      // convert subdevices to devices and remove duplicates
+      /* convert subdevices to devices and remove duplicates */
       cl_uint real_num_devices = 0;
       unique_devlist = pocl_unique_device_list (device_list, num_devices,
                                                 &real_num_devices);
@@ -781,7 +774,8 @@ compile_and_link_program(int compile_program,
     {
       cl_device_id device = program->devices[device_i];
 
-      if (cl_c_version && check_device_supports (device, cl_c_version))
+      if (!pocl_get_bool_option ("POCL_IGNORE_CL_STD", 0) && cl_c_version
+          && check_device_supports (device, cl_c_version))
         {
           APPEND_TO_BUILD_LOG_GOTO (
               build_error_code,
@@ -916,12 +910,14 @@ compile_and_link_program(int compile_program,
       setup_device_kernel_hashes (program);
     }
 
-  for (device_i = 0; device_i < program->num_devices; device_i++)
+  if (link_program)
     {
-      cl_device_id device = program->devices[device_i];
-
-      if (device->ops->post_build_program)
+      for (device_i = 0; device_i < program->num_devices; device_i++)
         {
+          cl_device_id device = program->devices[device_i];
+          if (!device->ops->post_build_program)
+            continue;
+
           errcode = device->ops->post_build_program (program, device_i);
           if (errcode != CL_SUCCESS)
             {
