@@ -92,12 +92,11 @@ void pocl_cache_program_path(char*        path,
   program_device_dir (path, program, device_i, "");
 }
 
-// required in llvm API
 void pocl_cache_program_bc_path(char*        program_bc_path,
                                 cl_program   program,
                                 unsigned     device_i) {
-    program_device_dir(program_bc_path, program,
-                       device_i, POCL_PROGRAM_BC_FILENAME);
+  program_device_dir (program_bc_path, program,
+                      device_i, POCL_PROGRAM_BC_FILENAME);
 }
 
 void
@@ -208,9 +207,9 @@ pocl_cache_kernel_cachedir (char *kernel_cachedir_path, cl_program program,
 {
   int bytes_written;
   char tempstring[POCL_MAX_PATHNAME_LENGTH];
-  char file_name[POCL_MAX_DIRNAME_LENGTH + 1];
+  char file_name[POCL_MAX_FILENAME_LENGTH + 1];
 
-  pocl_hash_clipped_name (kernel_name, POCL_MAX_DIRNAME_LENGTH, &file_name[0]);
+  pocl_hash_clipped_name (kernel_name, POCL_MAX_FILENAME_LENGTH, &file_name[0]);
 
   bytes_written
       = snprintf (tempstring, POCL_MAX_PATHNAME_LENGTH, "/%s", file_name);
@@ -371,7 +370,7 @@ pocl_cache_write_descriptor (_cl_command_node *command, cl_kernel kernel,
   if (pocl_mkdir_p (dirr))
     return -1;
 
-  return pocl_write_file (descriptor, content, size, 0, 1);
+  return pocl_write_file (descriptor, content, size, 0);
 }
 
 /******************************************************************************/
@@ -405,7 +404,7 @@ int pocl_cache_append_to_buildlog(cl_program  program,
     program_device_dir(buildlog_path, program,
                        device_i, POCL_BUILDLOG_FILENAME);
 
-    return pocl_write_file(buildlog_path, content, size, 1, 1);
+    return pocl_write_file(buildlog_path, content, size, 1);
 }
 
 /******************************************************************************/
@@ -431,7 +430,7 @@ pocl_cache_write_kernel_parallel_bc (void *bc, cl_program program,
   assert (strlen (kernel_parallel_path)
           < (POCL_MAX_PATHNAME_LENGTH - strlen (POCL_PARALLEL_BC_FILENAME)));
   strcat (kernel_parallel_path, POCL_PARALLEL_BC_FILENAME);
-  return pocl_write_module (bc, kernel_parallel_path, 0);
+  return pocl_write_module (bc, kernel_parallel_path);
 }
 #endif
 
@@ -480,7 +479,7 @@ build_program_compute_hash (cl_program program, unsigned device_i,
       }
 #endif
 
-#ifdef ENABLE_SPIR
+#ifdef ENABLE_SPIRV
     for (size_t i = 0; i < program->num_spec_consts; ++i)
       {
         if (program->spec_const_is_set[i])
