@@ -269,13 +269,13 @@ extern uint64_t last_object_id;
 #  define POCL_ALIAS_OPENCL_SYMBOL(name)                                \
   __typeof__(name) name __attribute__((alias ("PO" #name), visibility("default")));
 
-#  if !defined(BUILD_ICD) && !defined(BUILD_PROXY)
+#if !defined(BUILD_ICD) && !defined(RENAME_POCL)
 #    define POsym(name) POCL_ALIAS_OPENCL_SYMBOL(name)
 #  else
 #    define POsym(name)
 #  endif
 
-#  if !defined(BUILD_PROXY)
+#if !defined(RENAME_POCL)
 #    define POsymAlways(name) POCL_ALIAS_OPENCL_SYMBOL(name)
 #  else
 #    define POsymAlways(name)
@@ -324,8 +324,9 @@ typedef struct pocl_argument {
   /* 1 if the argument is read-only according to kernel metadata. So either
    * a buffer with "const" qualifier, or an image with read_only qualifier  */
   char is_readonly;
-  /* 1 if the argument pointer is SVM direct pointer, not a cl_mem */
-  char is_svm;
+  /* 1 if the argument pointer is a raw pointer (SVM, USM or device),
+     not a cl_mem handle */
+  char is_raw_ptr;
 } pocl_argument;
 
 typedef struct event_node event_node;
@@ -333,11 +334,13 @@ typedef struct event_node event_node;
 /**
  * Enumeration for kernel argument types
  */
-typedef enum {
+typedef enum
+{
   POCL_ARG_TYPE_NONE = 0,
   POCL_ARG_TYPE_POINTER = 1,
   POCL_ARG_TYPE_IMAGE = 2,
   POCL_ARG_TYPE_SAMPLER = 3,
+  POCL_ARG_TYPE_PIPE = 4,
 } pocl_argument_type;
 
 #define ARG_IS_LOCAL(a) (a.address_qualifier == CL_KERNEL_ARG_ADDRESS_LOCAL)
@@ -434,6 +437,8 @@ struct pocl_device_ops {
    *  parameters : optional environment with device-specific parameters
    */
   cl_int (*init) (unsigned j, cl_device_id device, const char *parameters);
+  /* Device type initialization after all devices have been initialized */
+  cl_int (*post_init) (struct pocl_device_ops *ops);
   /* uninitializes the driver for a particular device. May free hardware resources. */
   cl_int (*uninit) (unsigned j, cl_device_id device);
   /* reinitializes the driver for a particular device. Called after uninit;
@@ -478,6 +483,26 @@ struct pocl_device_ops {
                        size_t *__restrict__ sizes);
   void (*svm_advise) (cl_device_id dev, const void *svm_ptr, size_t size,
                       cl_mem_advice_intel advice);
+  /* required for PoCL's command buffer extensions */
+  void (*svm_copy_rect) (cl_device_id dev,
+                         void *__restrict__ dst,
+                         const void *__restrict__ src,
+                         const size_t *dst_origin,
+                         const size_t *src_origin,
+                         const size_t *region,
+                         size_t dst_row_pitch,
+                         size_t dst_slice_pitch,
+                         size_t src_row_pitch,
+                         size_t src_slice_pitch);
+  void (*svm_fill_rect) (cl_device_id dev,
+                         void *__restrict__ svm_ptr,
+                         const size_t *origin,
+                         const size_t *region,
+                         size_t row_pitch,
+                         size_t slice_pitch,
+                         void *__restrict__ pattern,
+                         size_t pattern_size);
+
   /* USM Ops (Intel) */
   void *(*usm_alloc) (cl_device_id dev, unsigned alloc_type,
                       cl_mem_alloc_flags_intel flags, size_t size, cl_int *errcode);
@@ -653,8 +678,8 @@ struct pocl_device_ops {
   int (*init_queue) (cl_device_id device, cl_command_queue queue);
   int (*free_queue) (cl_device_id device, cl_command_queue queue);
 
-  /* Optional. if the driver needs to use hardware resources
-   * for contexts, it should use these callbacks */
+  /* Optional. if the driver needs to use per-context resources,
+   * it should use these callbacks for management. */
   int (*init_context) (cl_device_id device, cl_context context);
   int (*free_context) (cl_device_id device, cl_context context);
 
@@ -780,6 +805,11 @@ struct pocl_device_ops {
                                       size_t param_value_size,
                                       const void *param_value);
 
+  /* optional. Returns synchronized Device & Host timestamps. */
+  cl_int (*get_synchronized_timestamps) (cl_device_id dev,
+                                         cl_ulong *dev_timestamp,
+                                         cl_ulong *host_timestamp);
+
   /* optional. Return CL_SUCCESS if the device can be, or is associated with
    * the GL context described in properties. */
   cl_int (*get_gl_context_assoc) (cl_device_id device, cl_gl_context_info type,
@@ -829,6 +859,7 @@ struct _cl_device_id {
   size_t preferred_wg_size_multiple;
   cl_bool non_uniform_work_group_support;
   cl_bool generic_as_support;
+  cl_bool wg_collective_func_support;
   cl_uint preferred_vector_width_char;
   cl_uint preferred_vector_width_short;
   cl_uint preferred_vector_width_int;
@@ -1050,8 +1081,6 @@ struct _cl_device_id {
   /* Device operations, shared among devices of the same type */
   struct pocl_device_ops *ops;
 
-  /* cl_khr_spir / CL_DEVICE_SPIR_VERSIONS, only includes SPIR not SPIR-V */
-  const char *supported_spir_versions;
   /* cl_khr_il_program / CL_DEVICE_IL_VERSION, this only includes SPIR-V */
   const char *supported_spir_v_versions;
 
@@ -1094,6 +1123,8 @@ struct _cl_device_id {
   cl_device_unified_shared_memory_capabilities_intel single_shared_usm_capabs;
   cl_device_unified_shared_memory_capabilities_intel cross_shared_usm_capabs;
   cl_device_unified_shared_memory_capabilities_intel system_shared_usm_capabs;
+
+  struct _cl_device_id *next;
 };
 
 #define DEVICE_SVM_FINEGR(dev) (dev->svm_caps & (CL_DEVICE_SVM_FINE_GRAIN_BUFFER \
@@ -1139,12 +1170,44 @@ struct _context_destructor_callback
   context_destructor_callback_t *next;
 };
 
-typedef struct _pocl_svm_ptr pocl_svm_ptr;
-struct _pocl_svm_ptr
+/**
+ * Enumeration for raw buffer/pointer types managed by PoCL.
+ */
+typedef enum
 {
-  void *svm_ptr;
+  /* SVM from OpenCL 2.0. */
+  POCL_RAW_PTR_SVM = 0,
+  /* Intel USM extension. */
+  POCL_RAW_PTR_INTEL_USM,
+  /* cl_ext_buffer_device_address. */
+  POCL_RAW_PTR_DEVICE_BUFFER
+} pocl_raw_pointer_kind;
+
+typedef struct _pocl_raw_ptr pocl_raw_ptr;
+struct _pocl_raw_ptr
+{
+  /* The virtual address, if any.  NULL if there's none. */
+  void *vm_ptr;
+  /* The device address, if known. NULL if not. */
+  void *dev_ptr;
+  /* The owner device of the allocation, if any. Should be set to non-null for
+     USM Device. */
+  cl_device_id device;
+
   size_t size;
-  struct _pocl_svm_ptr *prev, *next;
+  /* A cl_mem for internal bookkeeping and implicit buffer migration. */
+  cl_mem shadow_cl_mem;
+
+  /* The raw pointer/buffer API used for the allocation. */
+  pocl_raw_pointer_kind kind;
+
+  struct
+  {
+    cl_mem_alloc_flags_intel flags;
+    unsigned alloc_type;
+  } usm_properties;
+
+  struct _pocl_raw_ptr *prev, *next;
 };
 
 struct _cl_context {
@@ -1186,7 +1249,7 @@ struct _cl_context {
 
   /* for enqueueing migration commands. Two reasons:
    * 1) since migration commands can execute in parallel
-   * to other commands, we can increase paralelism
+   * to other commands, we can increase parallelism
    * 2) in some cases (migration between 2 devices through
    * host memory), we need to put two commands in two queues,
    * and the clEnqueueX only gives us one (on the destination
@@ -1201,11 +1264,12 @@ struct _cl_context {
    */
   size_t min_buffer_alignment;
 
-  /* list of destructor callbacks */
+  /* List of destructor callbacks */
   context_destructor_callback_t *destructor_callbacks;
 
-  /* list of SVM & USM allocations */
-  pocl_svm_ptr *svm_ptrs;
+  /* List of allocations with raw host-side accessible pointers associated
+     with them (SVM, USM, DEV). */
+  pocl_raw_ptr *raw_ptrs;
 
   /* list of command queues created for the context.
    * required for clMemBlockingFreeINTEL */
@@ -1243,6 +1307,10 @@ struct _cl_command_queue {
   cl_queue_properties queue_properties[10];
   unsigned num_queue_properties;
 
+  /* number of user threads (via clFinish) awaiting
+   * cmd-queue-finished notification (via ops->notify_cmdq_finished) */
+  unsigned notification_waiting_threads;
+
   /* device specific data */
   void *data;
 
@@ -1441,10 +1509,10 @@ struct _cl_mem {
    * we can make some assumptions and optimizations */
   cl_bool mem_host_ptr_is_permanent;
 
-  /* If the allocation was requested to be permanent on the device
-     global memory (until freed). This is set via CL_MEM_PINNED
-     flag of cl_pocl_pinned_buffers extension. */
-  cl_bool is_device_pinned;
+  /* If the allocation was requested to have a permanent device global memory
+     address (until freed). This is set via CL_MEM_DEVICE_ADDRESS flag of
+     the cl_ext_buffer_device_address extension. */
+  cl_bool has_device_address;
 
   /* Image flags */
   cl_bool                 is_image;
@@ -1602,6 +1670,13 @@ struct _cl_program {
   char *spec_const_is_set;
 };
 
+typedef struct _pocl_ptr_list_node pocl_ptr_list;
+struct _pocl_ptr_list_node
+{
+  void *ptr;
+  struct _pocl_ptr_list_node *prev, *next;
+};
+
 struct _cl_kernel {
   POCL_ICD_OBJECT
   POCL_OBJECT;
@@ -1637,6 +1712,16 @@ struct _cl_kernel {
   char *dyn_argument_storage;
   void **dyn_argument_offsets;
 
+  /* The SVM, USM or device allocations accessed by the kernel which were set
+     explicitly using clSetKernelExecInfo(). */
+  pocl_ptr_list *indirect_raw_ptrs;
+
+  /* Set to true, in case the kernel might access any of the raw buffers
+     indirectly. All USM indirect access flags will set this currently.
+     We should ensure at enqueue time that all of the known raw buffers
+     will be synchronized to the device. */
+  char can_access_all_raw_buffers_indirectly;
+
   /* for program's linked list of kernels */
   struct _cl_kernel *next;
 };
@@ -1686,8 +1771,9 @@ struct _cl_event {
   /* list of callback functions */
   event_callback_item *callback_list;
 
-  /* list of devices needing completion notification for this event */
+  /* list of devices needing completion notification of this event */
   event_node *notify_list;
+  /* events this event is dependent on */
   event_node *wait_list;
 
   /* OoO doesn't use sync points -> put used buffers here */
@@ -1714,6 +1800,7 @@ struct _cl_event {
    * lowered and memory freed if it's 0 */
   short release_mem_host_ptr_after;
 
+  short profiling_available;
 
   _cl_event *next;
   _cl_event *prev;
