@@ -101,7 +101,7 @@ pocl_fill_buffer_common (cl_command_buffer_khr command_buffer,
       errcode = pocl_check_event_wait_list (
           command_queue, num_items_in_wait_list, event_wait_list);
       if (errcode != CL_SUCCESS)
-          return errcode;
+        return errcode;
       errcode = pocl_create_command (
           cmd, command_queue, CL_COMMAND_FILL_BUFFER, event,
           num_items_in_wait_list, event_wait_list, 1, &buffer, &rdonly);
@@ -172,7 +172,8 @@ pocl_fill_image_common (cl_command_buffer_khr command_buffer,
   if (errcode != CL_SUCCESS)
     return errcode;
 
-  cl_uint4 fill_color_vec = *(const cl_uint4 *)fill_color;
+  cl_uint4 fill_color_vec;
+  memcpy(&fill_color_vec, fill_color, 16);
 
   size_t px = image->image_elem_size * image->image_channels;
   char fill_pattern[16];
@@ -218,7 +219,7 @@ pocl_fill_image_common (cl_command_buffer_khr command_buffer,
       errcode = pocl_check_event_wait_list (
           command_queue, num_items_in_wait_list, event_wait_list);
       if (errcode != CL_SUCCESS)
-          return errcode;
+        return errcode;
       errcode = pocl_create_command (cmd, command_queue, CL_COMMAND_FILL_IMAGE,
                                      event, num_items_in_wait_list,
                                      event_wait_list, 1, &image, &rdonly);
