@@ -35,9 +35,9 @@ extern "C" {
 POdeclsym(clBuildProgram)
 POdeclsym(clLinkProgram)
 POdeclsym(clCompileProgram)
-POdeclsym(clCreateBuffer)
+POdeclsymExport(clCreateBuffer)
 POdeclsym(clCreateCommandQueue)
-POdeclsym(clCreateContext)
+POdeclsymExport(clCreateContext)
 POdeclsym(clCreateContextFromType)
 POdeclsym(clCreateImage2D) 
 POdeclsym(clCreateImage3D)
@@ -93,7 +93,7 @@ POdeclsym(clGetKernelInfo)
 POdeclsym(clGetKernelArgInfo)
 POdeclsym(clGetKernelWorkGroupInfo)
 POdeclsym(clGetKernelSubGroupInfo)
-POdeclsym(clGetMemObjectInfo)
+POdeclsymExport(clGetMemObjectInfo)
 POdeclsym(clGetPlatformIDs)
 POdeclsym(clGetPlatformInfo)
 POdeclsym(clGetProgramBuildInfo)
@@ -176,6 +176,9 @@ POdeclsym(clCommandFillBufferKHR)
 POdeclsym(clCommandFillImageKHR)
 POdeclsym(clGetCommandBufferInfoKHR)
 
+/* cl_khr_command_buffer_multi_device */
+POdeclsym(clRemapCommandBufferKHR)
+
 /* cl_intel_unified_shared_memory */
 POdeclsym(clHostMemAllocINTEL)
 POdeclsym(clDeviceMemAllocINTEL)
@@ -193,6 +196,26 @@ POdeclsym(clEnqueueMigrateMemINTEL)
 POdeclsym(clCommandSVMMemcpyKHR)
 POdeclsym(clCommandSVMMemFillKHR)
 
+/* cl_pocl_command_buffer_svm */
+POdeclsym(clCommandSVMMemcpyPOCL)
+POdeclsym(clCommandSVMMemcpyRectPOCL)
+POdeclsym(clCommandSVMMemfillPOCL)
+POdeclsym(clCommandSVMMemfillRectPOCL)
+
+/* cl_pocl_command_buffer_host_buffer */
+POdeclsym(clCommandReadBufferPOCL)
+POdeclsym(clCommandReadBufferRectPOCL)
+POdeclsym(clCommandReadImagePOCL)
+POdeclsym(clCommandWriteBufferPOCL)
+POdeclsym(clCommandWriteBufferRectPOCL)
+POdeclsym(clCommandWriteImagePOCL)
+
+/* cl_pocl_svm_rect */
+POdeclsym(clEnqueueSVMMemFillRectPOCL)
+POdeclsym(clEnqueueSVMMemcpyRectPOCL)
+
+/* cl_ext_buffer_device_address */
+POdeclsym (clSetKernelArgDevicePointerEXT);
 
 #ifdef __cplusplus
 }
