@@ -179,7 +179,6 @@ static bool generateProgramBC(PoclLLVMContextData *Context, llvm::Module *Mod,
   if (unifyPrintfFingerPrint(Mod, BuiltinLib))
     return true;
 
-#ifndef LLVM_OLDER_THAN_14_0
   if (Device->run_program_scope_variables_pass) {
     size_t TotalGVarBytes = 0;
     if (runProgramScopeVariablesPass(Mod, Device->global_as_id,
@@ -187,7 +186,6 @@ static bool generateProgramBC(PoclLLVMContextData *Context, llvm::Module *Mod,
       return true;
     Program->global_var_total_size[device_i] = TotalGVarBytes;
   }
-#endif
 
   if (link(Mod, BuiltinLib, Log, Device->device_aux_functions,
            Device->device_side_printf != CL_FALSE))
@@ -251,7 +249,7 @@ int pocl_llvm_build_program(cl_program program,
         std::string dir(path, 0, (tempdir.size() + 1 + last_slash));
         pocl_mkdir_p(dir.c_str());
       }
-      pocl_write_file(path.c_str(), input_header, input_header_size, 0, 1);
+      pocl_write_file(path.c_str(), input_header, input_header_size, 0);
     }
   }
   // Use CompilerInvocation::CreateFromArgs to initialize
@@ -315,7 +313,7 @@ int pocl_llvm_build_program(cl_program program,
   // required for clGetKernelArgInfo()
   ss << "-cl-kernel-arg-info ";
 
-#if (CLANG_MAJOR == 15) || (CLANG_MAJOR == 16)
+#if (LLVM_MAJOR == 15) || (LLVM_MAJOR == 16)
 #ifdef LLVM_OPAQUE_POINTERS
   ss << "-opaque-pointers ";
 #else
@@ -405,7 +403,10 @@ int pocl_llvm_build_program(cl_program program,
     ss << "-cl-ext=-all," << cl_ext;
   }
 
+  // do not use LLVM builtin functions, rely on PoCL bitcode library only
   ss << "-fno-builtin ";
+  // do not use jump/switch tables, these create a problem for VUA pass
+  ss << "-fno-jump-tables ";
 
   // This is required otherwise the initialization fails with
   // unknown triple ''
@@ -421,10 +422,10 @@ int pocl_llvm_build_program(cl_program program,
 
   char WSReplacementChar = 0;
 
-  char *TempOptions = (char *) malloc (ss.str().length() + 1);
+  char *TempOptions = (char *)malloc(AllBuildOpts.length() + 1);
 
-  memset (TempOptions, 0, ss.str().length() + 1);
-  strncpy (TempOptions, ss.str().c_str(), ss.str().length());
+  memset(TempOptions, 0, AllBuildOpts.length() + 1);
+  strncpy(TempOptions, AllBuildOpts.c_str(), AllBuildOpts.length());
 
   if (pocl_escape_quoted_whitespace (TempOptions, &WSReplacementChar) == -1)
   {
@@ -459,6 +460,14 @@ int pocl_llvm_build_program(cl_program program,
     // There can be space even without quotes in the user input.
     s = std::regex_replace(s, std::regex("-I(\\s+)"), "-I");
 
+    // Convert the -g to more specific debug options understood by CFE.
+    if (s == "-g") {
+      itemstrs.push_back("-debug-info-kind=limited");
+      itemstrs.push_back("-dwarf-version=4");
+      itemstrs.push_back("-debugger-tuning=gdb");
+      continue;
+    }
+
     itemstrs.push_back(s);
   }
 
@@ -471,8 +480,10 @@ int pocl_llvm_build_program(cl_program program,
   POCL_MEM_FREE (TempOptions);
 
 #ifdef DEBUG_POCL_LLVM_API
-  std::cerr << "### options: " << ss.str() << std::endl << std::endl
-            << "user_options: " << user_options << std::endl << std::endl
+  std::cerr << "### options: " << AllBuildOpts << std::endl
+            << std::endl
+            << "user_options: " << user_options << std::endl
+            << std::endl
             << "c_strs: ";
   for (auto cstr : itemcstrs) {
     std::cerr << cstr << " ";
@@ -493,22 +504,23 @@ int pocl_llvm_build_program(cl_program program,
     return CL_INVALID_BUILD_OPTIONS;
   }
 
+#if LLVM_MAJOR < 18
   LangOptions *la = pocl_build.getLangOpts();
+#else
+  LangOptions L = pocl_build.getLangOpts();
+  LangOptions *la = &L;
+#endif
   PreprocessorOptions &po = pocl_build.getPreprocessorOpts();
   llvm::Triple triple (device->llvm_target_triplet);
 
-#ifndef LLVM_OLDER_THAN_15_0
+#if LLVM_MAJOR >= 15
   LangOptions::setLangDefaults(*la, clang::Language::OpenCL, triple,
                                po.Includes, clang::LangStandard::lang_opencl12);
 #else
   pocl_build.setLangDefaults(*la,
                              clang::InputKind(clang::Language::OpenCL),
                              triple,
-#ifndef LLVM_OLDER_THAN_12_0
                              po.Includes,
-#else
-                             po,
-#endif
                              clang::LangStandard::lang_opencl12);
 #endif
 
@@ -528,23 +540,11 @@ int pocl_llvm_build_program(cl_program program,
   // setLangDefaults overrides to FPM_On for OpenCL.
   // So, we need to manually set it after
   if (fp_contract == "fast") {
-#ifndef LLVM_OLDER_THAN_11_0
     la->setDefaultFPContractMode(LangOptions::FPM_Fast);
-#else
-    la->setDefaultFPContractMode(LangOptions::FPC_Fast);
-#endif
   } else if (fp_contract == "on") {
-#ifndef LLVM_OLDER_THAN_11_0
     la->setDefaultFPContractMode(LangOptions::FPM_On);
-#else
-    la->setDefaultFPContractMode(LangOptions::FPC_On);
-#endif
   } else if (fp_contract == "off") {
-#ifndef LLVM_OLDER_THAN_11_0
     la->setDefaultFPContractMode(LangOptions::FPM_Off);
-#else
-    la->setDefaultFPContractMode(LangOptions::FPC_Off);
-#endif
   }
 
   la->setStackProtector(LangOptions::StackProtectorMode::SSPOff);
@@ -573,7 +573,7 @@ int pocl_llvm_build_program(cl_program program,
 #endif
   }
   if (ClangResourceDir.empty()) {
-    ClangResourceDir = driver::Driver::GetResourcesPath(CLANG);
+    ClangResourceDir = clang::driver::Driver::GetResourcesPath(CLANG);
   }
   KernelH = IncludeRoot + "/include/_kernel.h";
   BuiltinRenamesH = IncludeRoot + "/include/_builtin_renames.h";
@@ -756,7 +756,7 @@ int pocl_llvm_build_program(cl_program program,
   POCL_MSG_PRINT_LLVM("Writing program.bc to %s.\n", program_bc_path);
 
   /* Always retain program.bc */
-  error = pocl_write_module(mod, program_bc_path, 0);
+  error = pocl_write_module(mod, program_bc_path);
   if(error)
     return error;
 
@@ -784,7 +784,6 @@ static int pocl_convert_spir_bitcode_to_target(llvm::Module *p,
                                                cl_device_id device) {
   const std::string &ModTriple = p->getTargetTriple();
   if (ModTriple.find("spir") == 0) {
-#ifdef ENABLE_SPIR
     POCL_RETURN_ERROR_ON((device->endian_little == CL_FALSE),
                          CL_LINK_PROGRAM_FAILURE,
                          "SPIR is only supported on little-endian devices\n");
@@ -808,10 +807,6 @@ static int pocl_convert_spir_bitcode_to_target(llvm::Module *p,
     if (p->getModuleFlag("PIC Level") == nullptr)
       p->setPICLevel(PICLevel::BigPIC);
     return CL_SUCCESS;
-#else
-    POCL_MSG_ERR("SPIR not supported\n");
-    return CL_LINK_PROGRAM_FAILURE;
-#endif
   }
   return CL_SUCCESS;
 }
@@ -857,8 +852,8 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
       TempModule = llvm::CloneModule(*Ptr);
     } else {
       llvm::Module *Ptr =
-          parseModuleIRMem((char *)cur_device_binaries[0],
-                           cur_device_binary_sizes[0], llvm_ctx->Context);
+          parseModuleIRMem((char *)cur_device_binaries[i],
+                           cur_device_binary_sizes[i], llvm_ctx->Context);
       POCL_RETURN_ERROR_ON((Ptr == nullptr), CL_LINK_PROGRAM_FAILURE,
                            "could not parse module\n");
       TempModule.reset(Ptr);
@@ -928,7 +923,7 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
   POCL_MSG_PRINT_LLVM("Writing program.bc to %s.\n", program_bc_path);
 
   /* Always retain program.bc for metadata */
-  error = pocl_write_module(LinkedModule, program_bc_path, 0);
+  error = pocl_write_module(LinkedModule, program_bc_path);
   if (error)
     return error;
 
@@ -1032,18 +1027,18 @@ int pocl_invoke_clang(cl_device_id Device, const char** Args) {
 
   DiagnosticsEngine Diags(DiagID, &*DiagOpts, DiagClient);
 
-  driver::Driver TheDriver(CLANG, Device->llvm_target_triplet, Diags);
+  clang::driver::Driver TheDriver(CLANG, Device->llvm_target_triplet, Diags);
 
   const char **ArgsEnd = Args;
   while (*ArgsEnd++ != nullptr) {}
 
   llvm::ArrayRef<const char*> ArgsArray(Args, ArgsEnd);
 
-  std::unique_ptr<driver::Compilation> C(
-    TheDriver.BuildCompilation(ArgsArray));
+  std::unique_ptr<clang::driver::Compilation> C(
+      TheDriver.BuildCompilation(ArgsArray));
 
   if (C && !C->containsError()) {
-    SmallVector<std::pair<int, const driver::Command *>, 4> FailingCommands;
+    SmallVector<std::pair<int, const clang::driver::Command *>, 4> FailingCommands;
     return TheDriver.ExecuteCompilation(*C, FailingCommands);
   } else {
     return -1;
