@@ -105,15 +105,14 @@ void writeModuleIRtoString(const llvm::Module *mod, std::string& dest) {
   sos.str(); // flush
 }
 
-int pocl_write_module(void *module, const char *path, int dont_rewrite) {
+int pocl_write_module(void *module, const char *path) {
   assert(module);
   assert(path);
 
   std::string binary;
   writeModuleIRtoString((const Module *)module, binary);
 
-  return pocl_write_file(path, binary.data(), (uint64_t)binary.size(), 0,
-                         dont_rewrite);
+  return pocl_write_file(path, binary.data(), (uint64_t)binary.size(), 0);
 }
 
 llvm::Module *parseModuleIRMem(const char *input_stream, size_t size,
@@ -123,8 +122,11 @@ llvm::Module *parseModuleIRMem(const char *input_stream, size_t size,
       MemoryBuffer::getMemBufferCopy(input_stream_ref);
 
   auto parsed_module = parseBitcodeFile(buffer->getMemBufferRef(), *c);
-  if (!parsed_module)
+  if (auto error = parsed_module.takeError()) {
+    POCL_MSG_ERR("parseBitcodeFile failed:\n%s\n",
+                 toString(std::move(error)).c_str());
     return nullptr;
+  }
   return parsed_module.get().release();
 }
 
@@ -254,7 +256,7 @@ const struct kernellib_features {
 const char *pocl_get_distro_kernellib_variant() {
   StringMap<bool> Features;
 
-#if defined(__x86_64__)
+#if defined(__i386__) || defined(__x86_64__)
   if (!llvm::sys::getHostCPUFeatures(Features)) {
     POCL_MSG_WARN("LLVM can't get host CPU flags!\n");
     return NULL;
@@ -292,7 +294,7 @@ const char *pocl_get_distro_kernellib_variant() {
 const char *pocl_get_distro_cpu_name(const char *kernellib_variant) {
   StringMap<bool> Features;
 
-#if defined(__x86_64__)
+#if defined(__i386__) || defined(__x86_64__)
   if (!llvm::sys::getHostCPUFeatures(Features)) {
     POCL_MSG_WARN("LLVM can't get host CPU flags!\n");
     return NULL;
@@ -357,14 +359,14 @@ void cpu_setup_vector_widths(cl_device_id dev) {
       VECWIDTH(cl_long);
   dev->native_vector_width_float = dev->preferred_vector_width_float =
       VECWIDTH(float);
-  if (strstr(HOST_DEVICE_EXTENSIONS, "cl_khr_fp64") == NULL) {
+  if (strstr(dev->extensions, "cl_khr_fp64") == NULL) {
     dev->native_vector_width_double = dev->preferred_vector_width_double = 0;
   } else {
     dev->native_vector_width_double = dev->preferred_vector_width_double =
         VECWIDTH(double);
   }
 
-  if (strstr(HOST_DEVICE_EXTENSIONS, "cl_khr_fp16") == NULL) {
+  if (strstr(dev->extensions, "cl_khr_fp16") == NULL) {
     dev->native_vector_width_half = dev->preferred_vector_width_half = 0;
   } else {
     dev->native_vector_width_half = dev->preferred_vector_width_half =
@@ -443,7 +445,7 @@ void InitializeLLVM() {
     initializeAnalysis(Registry);
     initializeTransformUtils(Registry);
     initializeInstCombine(Registry);
-#ifdef LLVM_OLDER_THAN_16_0
+#if LLVM_MAJOR < 16
     initializeInstrumentation(Registry);
 #endif
     initializeTarget(Registry);
@@ -527,11 +529,6 @@ void InitializeLLVM() {
       assert(O && "could not find LLVM option 'debug'");
       O->addOccurrence(1, StringRef("debug"), StringRef("true"), false);
     }
-#if LLVM_MAJOR == 9
-    O = opts["unroll-threshold"];
-    assert(O && "could not find LLVM option 'unroll-threshold'");
-    O->addOccurrence(1, StringRef("unroll-threshold"), StringRef("1"), false);
-#endif
   }
 }
 
@@ -562,7 +559,7 @@ void pocl_llvm_create_context(cl_context ctx) {
 
   data->Context = new llvm::LLVMContext();
   assert(data->Context);
-#if (CLANG_MAJOR == 15) || (CLANG_MAJOR == 16)
+#if (LLVM_MAJOR == 15) || (LLVM_MAJOR == 16)
 #ifdef LLVM_OPAQUE_POINTERS
   data->Context->setOpaquePointers(true);
 #else
@@ -634,6 +631,28 @@ void pocl_llvm_release_context(cl_context ctx) {
   }
 }
 
+void pocl_append_to_buildlog(cl_program Program, cl_uint DeviceI, char *Log,
+                             size_t LogSize) {
+  size_t ExistingLogSize = 0;
+  if (LogSize == 0) {
+    return;
+  }
+
+  if (Program->build_log[DeviceI] != nullptr) {
+    ExistingLogSize = strlen(Program->build_log[DeviceI]);
+    size_t TotalLogSize = LogSize + ExistingLogSize;
+    char *NewLog = (char *)malloc(TotalLogSize);
+    assert(NewLog);
+    memcpy(NewLog, Program->build_log[DeviceI], ExistingLogSize);
+    memcpy(NewLog + ExistingLogSize, Log, LogSize);
+    free(Log);
+    free(Program->build_log[DeviceI]);
+    Program->build_log[DeviceI] = NewLog;
+  } else {
+    Program->build_log[DeviceI] = Log;
+  }
+}
+
 #define POCL_METADATA_ROOT "pocl_meta"
 
 void setModuleIntMetadata(llvm::Module *mod, const char *key, unsigned long data) {
