@@ -117,6 +117,13 @@ _cl_command_node* pocl_mem_manager_new_command ()
 
 void pocl_mem_manager_free_command (_cl_command_node *cmd_ptr)
 {
+  if (cmd && cmd->buffered)
+    {
+      /* TODO: recycle these somehow? */
+      POCL_MEM_FREE (cmd->sync.syncpoint.sync_point_wait_list);
+      POCL_MEM_FREE (cmd->memobj_list);
+      POCL_MEM_FREE (cmd->readonly_flag_list);
+    }
   POCL_LOCK (mm->cmd_lock);
   LL_PREPEND (mm->cmd_list, cmd_ptr);
   POCL_UNLOCK(mm->cmd_lock);
