@@ -1,6 +1,6 @@
 /* pocl_ndrange_kernel.c: helpers for NDRange Kernel commands
 
-   Copyright (c) 2022 Jan Solanti / Tampere University
+   Copyright (c) 2022-2024 Jan Solanti / Tampere University
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
@@ -24,6 +24,7 @@
 #include "pocl_cl.h"
 #include "pocl_local_size.h"
 #include "pocl_mem_management.h"
+#include "pocl_shared.h"
 #include "pocl_util.h"
 
 #include <assert.h>
@@ -49,8 +50,6 @@ pocl_kernel_calc_wg_size (cl_command_queue command_queue, cl_kernel kernel,
    * since we are going to access them repeatedly */
   size_t max_group_size;
 
-  cl_int errcode = CL_SUCCESS;
-
   POCL_RETURN_ERROR_COND ((work_dim < 1), CL_INVALID_WORK_DIMENSION);
   POCL_RETURN_ERROR_ON (
       (work_dim > command_queue->device->max_work_item_dimensions),
@@ -211,8 +210,11 @@ SKIP_WG_SIZE_CALCULATION:
 }
 
 static cl_int
-pocl_kernel_collect_mem_objs (cl_command_queue command_queue, cl_kernel kernel,
-                              cl_uint *memobj_count, cl_mem *memobj_list,
+pocl_kernel_collect_mem_objs (cl_command_queue command_queue,
+                              cl_kernel kernel,
+                              struct pocl_argument *src_arguments,
+                              cl_uint *memobj_count,
+                              cl_mem *memobj_list,
                               char *readonly_flag_list)
 {
   cl_device_id realdev = pocl_real_dev (command_queue->device);
@@ -220,17 +222,44 @@ pocl_kernel_collect_mem_objs (cl_command_queue command_queue, cl_kernel kernel,
   for (unsigned i = 0; i < kernel->meta->num_args; ++i)
     {
       struct pocl_argument_info *a = &kernel->meta->arg_info[i];
-      struct pocl_argument *al = &(kernel->dyn_arguments[i]);
+      struct pocl_argument *al = &(src_arguments[i]);
 
       POCL_RETURN_ERROR_ON ((!al->is_set), CL_INVALID_KERNEL_ARGS,
                             "The %i-th kernel argument is not set!\n", i);
 
-      if (a->type == POCL_ARG_TYPE_IMAGE
+      if (a->type == POCL_ARG_TYPE_IMAGE || a->type == POCL_ARG_TYPE_PIPE
           || (!ARGP_IS_LOCAL (a) && a->type == POCL_ARG_TYPE_POINTER
-              && al->value != NULL && al->is_svm == 0))
+              && al->value != NULL))
         {
           cl_mem buf = *(cl_mem *)(al->value);
+          if (al->is_raw_ptr)
+            {
+              /* Find the shadow cl_mem wrapper which is used for tracking
+                 migrations. */
+              void *ptr = *(void **)(al->value);
+              pocl_raw_ptr *raw_ptr = pocl_find_raw_ptr_with_vm_ptr (
+                  command_queue->context, ptr);
 
+              /* TODO: The case where some of the args are SVM-allocated VM
+                 pointers, some device pointers. These address spaces are
+                 allowed to overlap and we could have the SVM and dev buffer
+                 having the same addr in theory. */
+              if (raw_ptr == NULL)
+                raw_ptr = pocl_find_raw_ptr_with_dev_ptr (
+                    command_queue->context, ptr);
+              if (raw_ptr == NULL)
+                {
+                  POCL_MSG_PRINT_MEMORY (
+                      "Couldn't find the shadow cl_mem for an SVM ptr, "
+                      "assuming system SVM.\n");
+                  continue;
+                }
+              else
+                buf = raw_ptr->shadow_cl_mem;
+            }
+
+          if (buf == NULL) /* Likely USM. */
+            continue;
           if (a->type == POCL_ARG_TYPE_IMAGE)
             {
               POCL_RETURN_ON_UNSUPPORTED_IMAGE (buf, realdev);
@@ -242,7 +271,8 @@ pocl_kernel_collect_mem_objs (cl_command_queue command_queue, cl_kernel kernel,
 
               if (al->offset > 0)
                 POCL_RETURN_ERROR_ON (
-                    (al->offset % realdev->mem_base_addr_align != 0),
+                    (!buf->has_device_address
+                     && al->offset % realdev->mem_base_addr_align != 0),
                     CL_MISALIGNED_SUB_BUFFER_OFFSET,
                     "SubBuffer is not properly aligned for this device");
 
@@ -267,6 +297,47 @@ pocl_kernel_collect_mem_objs (cl_command_queue command_queue, cl_kernel kernel,
         }
     }
 
+  /* If the kernel has a general indirect access set, we append the
+     currently-alive raw buffers to the indirect_raw_buffers set and ensure
+     their data gets synchronized to the device. */
+  if (kernel->can_access_all_raw_buffers_indirectly)
+    {
+      struct _pocl_raw_ptr *ptr;
+      DL_FOREACH (kernel->context->raw_ptrs, ptr)
+      {
+        int found = 0;
+        /* Ensure we do not add the argument buffers again. */
+        for (cl_uint i = 0; i < *memobj_count; ++i)
+          if (memobj_list[i] == ptr->shadow_cl_mem)
+            {
+              found = 1;
+              break;
+            }
+        if (found)
+          continue;
+        memobj_list[(*memobj_count)++] = ptr->shadow_cl_mem;
+      }
+    }
+  else
+    {
+      /* Otherwise, we only migrate buffers related to USM/SVM pointers
+         explicitly set with clSetKernelExecInfo(). */
+      struct _pocl_ptr_list_node *n;
+      DL_FOREACH (kernel->indirect_raw_ptrs, n)
+      {
+        pocl_raw_ptr *svm_ptr
+          = pocl_find_raw_ptr_with_vm_ptr (command_queue->context, n->ptr);
+
+        if (svm_ptr == NULL)
+          {
+            POCL_MSG_PRINT_MEMORY ("Couldn't find the shadow cl_mem for an "
+                                   "clSetKernelExecInfo-set SVM ptr, "
+                                   "assuming system SVM.\n");
+            continue;
+          }
+        memobj_list[(*memobj_count)++] = svm_ptr->shadow_cl_mem;
+      }
+    }
   return CL_SUCCESS;
 }
 
@@ -309,32 +380,126 @@ pocl_kernel_copy_args (cl_kernel kernel,
   return CL_SUCCESS;
 }
 
+static int
+process_command_ndrange_properties (
+  const cl_ndrange_kernel_command_properties_khr *properties)
+{
+  if (properties == NULL)
+    return CL_SUCCESS;
+
+  cl_uint num_properties = 0;
+  const cl_ndrange_kernel_command_properties_khr *key = NULL;
+  for (key = properties; *key != 0; key += 2)
+    num_properties += 1;
+  POCL_RETURN_ERROR_ON ((num_properties == 0), CL_INVALID_VALUE,
+                        "Properties != NULL, but zero properties in array\n");
+  unsigned i = 0;
+  for (key = properties; *key != 0; key += 2, ++i)
+    {
+      switch (*key)
+        {
+        case CL_MUTABLE_DISPATCH_ASSERTS_KHR:
+        case CL_MUTABLE_DISPATCH_UPDATABLE_FIELDS_KHR:
+          POCL_RETURN_ERROR_ON (
+            1, CL_INVALID_VALUE,
+            "cl_khr_command_buffer_mutable_dispatch is not supported\n");
+          break;
+        default:
+          POCL_RETURN_ERROR_ON (1, CL_INVALID_VALUE,
+                                "Unknown property value in "
+                                "cl_ndrange_kernel_command_properties_khr\n");
+        }
+    }
+
+  return CL_SUCCESS;
+}
+
+cl_int
+pocl_record_ndrange_kernel (
+  cl_command_buffer_khr command_buffer,
+  cl_command_queue command_queue,
+  const cl_ndrange_kernel_command_properties_khr *properties,
+  cl_kernel kernel,
+  struct pocl_argument *src_arguments,
+  cl_uint work_dim,
+  const size_t *global_work_offset,
+  const size_t *global_work_size,
+  const size_t *local_work_size,
+  cl_uint num_items_in_wait_list,
+  const cl_sync_point_khr *sync_point_wait_list,
+  cl_sync_point_khr *sync_point_p)
+{
+  _cl_command_node *cmd = NULL;
+
+  int errcode = process_command_ndrange_properties (properties);
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
+
+  errcode = pocl_ndrange_kernel_common (
+    command_buffer, command_queue, properties, kernel, src_arguments, work_dim,
+    global_work_offset, global_work_size, local_work_size,
+    num_items_in_wait_list, NULL, NULL, sync_point_wait_list, sync_point_p,
+    &cmd);
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
+
+  for (unsigned i = 0; i < kernel->meta->num_args; ++i)
+    {
+      struct pocl_argument_info *ai
+        = &cmd->command.run.kernel->meta->arg_info[i];
+      struct pocl_argument *a = &cmd->command.run.kernel->dyn_arguments[i];
+      if (ai->type == POCL_ARG_TYPE_SAMPLER)
+        POname (clRetainSampler) (cmd->command.run.arguments[i].value);
+    }
+
+  errcode = pocl_command_record (command_buffer, cmd, sync_point_p);
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
+
+  return CL_SUCCESS;
+
+ERROR:
+  pocl_mem_manager_free_command (cmd);
+  return errcode;
+}
+
 cl_int
 pocl_ndrange_kernel_common (
-    cl_command_buffer_khr command_buffer, cl_command_queue command_queue,
-    const cl_ndrange_kernel_command_properties_khr *properties,
-    cl_kernel kernel, cl_uint work_dim, const size_t *global_work_offset,
-    const size_t *global_work_size, const size_t *local_work_size,
-    cl_uint num_items_in_wait_list, const cl_event *event_wait_list,
-    cl_event *event_p, const cl_sync_point_khr *sync_point_wait_list,
-    cl_sync_point_khr *sync_point_p, _cl_command_node **cmd)
+  cl_command_buffer_khr command_buffer,
+  cl_command_queue command_queue,
+  const cl_ndrange_kernel_command_properties_khr *properties,
+  cl_kernel kernel,
+  struct pocl_argument *src_arguments,
+  cl_uint work_dim,
+  const size_t *global_work_offset,
+  const size_t *global_work_size,
+  const size_t *local_work_size,
+  cl_uint num_items_in_wait_list,
+  const cl_event *event_wait_list,
+  cl_event *event_p,
+  const cl_sync_point_khr *sync_point_wait_list,
+  cl_sync_point_khr *sync_point_p,
+  _cl_command_node **cmd)
 {
   size_t offset[3] = { 0, 0, 0 };
   size_t num_groups[3] = { 0, 0, 0 };
   size_t local[3] = { 0, 0, 0 };
-  /* cached values for max_work_item_sizes,
-   * since we are going to access them repeatedly */
-  size_t max_local_x, max_local_y, max_local_z;
-  /* cached values for max_work_group_size,
-   * since we are going to access them repeatedly */
-  size_t max_group_size;
 
   int errcode = 0;
 
-  /* no need for malloc, pocl_create_event will memcpy anyway.
-   * num_args is the absolute max needed */
   cl_uint memobj_count = 0;
-  cl_mem memobj_list[kernel->meta->num_args];
+
+  size_t raw_ptr_count = 0;
+
+  /* At the worst case, we need to synchronize all raw buffers. */
+  struct _pocl_raw_ptr *ptr;
+  DL_FOREACH (kernel->context->raw_ptrs, ptr)
+    {
+      ++raw_ptr_count;
+    }
+
+  /* The list of memobjects to implicitly synchronize at kernel exec. */
+  cl_mem memobj_list[kernel->meta->num_args + raw_ptr_count];
   char readonly_flag_list[kernel->meta->num_args];
 
   assert (command_buffer == NULL
@@ -370,8 +535,9 @@ pocl_ndrange_kernel_common (
   POCL_RETURN_ERROR_ON (errcode != CL_SUCCESS, errcode,
                         "Error calculating wg size\n");
 
-  errcode = pocl_kernel_collect_mem_objs (command_queue, kernel, &memobj_count,
-                                          memobj_list, readonly_flag_list);
+  errcode = pocl_kernel_collect_mem_objs (command_queue, kernel, src_arguments,
+                                          &memobj_count, memobj_list,
+                                          readonly_flag_list);
   POCL_RETURN_ERROR_ON (errcode != CL_SUCCESS, errcode,
                         "Error collecting mem objects for kernel arguments\n");
 
@@ -414,8 +580,8 @@ pocl_ndrange_kernel_common (
   if (errcode != CL_SUCCESS)
     goto ERROR;
 
-  errcode = pocl_kernel_copy_args (kernel, kernel->dyn_arguments,
-                                   &(*cmd)->command.run);
+  errcode
+    = pocl_kernel_copy_args (kernel, src_arguments, &(*cmd)->command.run);
   if (errcode != CL_SUCCESS)
     goto ERROR;
 
