@@ -71,9 +71,8 @@ args = parser.parse_args()
 # function prefix used by PoCL's kernel library
 POCL_LIB_PREFIX = "_cl_"
 
-# CUDA uses the same pointer AS-mangling as SPIR
-# set to False for CUDA wrapper, True for CPU wrapper
-MANGLE_TARGET = args.target.startswith('cpu')
+# uses the same pointer AS-mangling as SPIR
+MANGLE_TARGET = True
 
 # if True, creates AS casts of pointer arguments (all are cast to AS 0)
 # set to False for CUDA wrapper, True for CPU wrapper
@@ -118,6 +117,10 @@ SINGLE_ARG = [
 	"native_log", "native_log2", "native_log10",
 	"native_recip", "native_rsqrt",
 	"native_sin", "native_sqrt", "native_tan",
+	"half_exp", "half_exp2","half_exp10",
+	"half_log", "half_log10", "half_log2",
+	"half_cos", "half_recip", "half_rsqrt",
+	"half_sin", "half_sqrt", "half_tan",
 	"degrees", "radians", "sign"
 ]
 
@@ -131,6 +134,7 @@ DUAL_ARG = [
 	"fdim", "fmax", "fmax_common", "fmin", "fmin_common", "fmod",
 	"hypot", "nextafter", "pow", "powr",
 	"maxmag", "minmag", "remainder",
+	"half_divide", "half_powr",
 	"native_divide", "native_powr",
 	"max", "min", "step"
 ]
@@ -172,11 +176,14 @@ OLD_ATOMICS_INT64_ONLY = [
 
 
 SVM_ATOMICS_INT_ONLY = [
-	"atomic_fetch_add",
-	"atomic_fetch_sub",
 	"atomic_fetch_or",
 	"atomic_fetch_xor",
 	"atomic_fetch_and",
+]
+
+SVM_ATOMICS_INT_AND_FLOAT = [
+	"atomic_fetch_add",
+	"atomic_fetch_sub",
 	"atomic_fetch_min",
 	"atomic_fetch_max"
 ]
@@ -526,22 +533,45 @@ MANGLING_AS_SPIR = {
 	"none": ""
 }
 
-MANGLING_AS_OCL = {
-	"global": "PU8CLglobal",
-	"constant": "PU10CLconstant",
-	"local": "PU7CLlocal",
-	"private": "PU9CLprivate",
-	"generic": "PU9CLgeneric",
-	"none": ""
-}
+if args.target.startswith('cpu'):
+	MANGLING_AS_OCL = {
+		"global": "PU8CLglobal",
+		"constant": "PU10CLconstant",
+		"local": "PU7CLlocal",
+		"private": "PU9CLprivate",
+		"generic": "PU9CLgeneric",
+		"none": ""
+	}
+else:
+	MANGLING_AS_OCL = {
+		"global": "PU3AS1",
+		"constant": "PU3AS4",
+		"local": "PU3AS3",
+		"private": "P",
+		"generic": "P",
+		"none": ""
+	}
 
-LLVM_SPIR_AS = {
-	"global": " addrspace(1)",
-	"constant": " addrspace(2)",
-	"local": " addrspace(3)",
-	"generic": " addrspace(4)",
-	"private": " ",
-	"none": " "
+if args.target.startswith('cpu'):
+	LLVM_SPIR_AS = {
+		"global": " addrspace(1)",
+		"constant": " addrspace(2)",
+		"local": " addrspace(3)",
+		"generic": " addrspace(4)",
+		"private": " ",
+		"none": " "
+	}
+else:
+	LLVM_SPIR_AS = {
+		"global": " addrspace(1)",
+		"constant": " addrspace(4)",
+		"local": " addrspace(3)",
+		"generic": " ",
+		"private": " ",
+		"none": " "
+	}
+
+ALREADY_DECLARED = {
 }
 
 # some coerced args require a shuffle b/c different size in bits
@@ -678,9 +708,14 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			addr_spaces = ["global", "local", "private", "constant"]
 		else:
 			addr_spaces = ["global", "local", "private"]
+		# "local" because some functions that have "global" only don't have generic versions
 		if GENERIC_AS and "local" in addr_spaces:
 			addr_spaces.append("generic")
 
+	if SPIR_CALLING_ABI and ("generic" in addr_spaces):
+		if not ("private" in addr_spaces):
+			addr_spaces.append("private")
+
 	for AS in addr_spaces:
 
 		spir_mangled_func_suffix = []
@@ -824,7 +859,14 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			decl_ret_type = 'void'
 		else:
 			decl_ret_type = coerced_ret_type
-		print("declare %s %s(%s) local_unnamed_addr #0" % (decl_ret_type, ocl_mangled_name, decl_args))
+		
+		#if GENERIC_AS and GEN_AS_CALLEE_IDENTICAL and (AS == "generic") and ("private" in addr_spaces):
+		declaration = "declare %s %s(%s) local_unnamed_addr #0" % (decl_ret_type, ocl_mangled_name, decl_args)
+		if declaration in ALREADY_DECLARED.keys():
+			print("")
+		else:
+			print(declaration)
+			ALREADY_DECLARED[declaration] = True
 		print("")
 		print("define spir_func %s %s(%s) local_unnamed_addr #0 {" % (ret_type_ext + ret_type, spir_mangled_name, caller_args))
 
@@ -1340,6 +1382,15 @@ for mang_type in ['i', 'j', "l", "m"]:
 		generate_function("atom_inc", ret_type, ret_ext, True, 'PV'+mang_type)
 		generate_function("atom_dec", ret_type, ret_ext, True, 'PV'+mang_type)
 
+for mang_type in ['i', 'j', "l", "m", 'f', 'd']:
+	ret_type = SIG_TO_LLVM_TYPE_MAP[mang_type]
+	ret_ext = LLVM_TYPE_EXT_MAP[mang_type]
+	for f in SVM_ATOMICS_INT_AND_FLOAT:
+		generate_function(f, ret_type, ret_ext, True, 'PVA'+mang_type, mang_type)
+		generate_function(f+"_explicit", ret_type, ret_ext, True, 'PVA'+mang_type, mang_type, "12memory_order")
+		generate_function(f+"_explicit", ret_type, ret_ext, True, 'PVA'+mang_type, mang_type, "12memory_order", "12memory_scope")
+
+
 # workaround for code generated by LLVM-SPIRV (with target-env == CL1.2,
 # emulates atomic_load/atomic_store with atomic_xchg/atomic_add)
 for mang_type in ['i', 'j', "f", ]:
@@ -1388,6 +1439,22 @@ for mang_type in ['i', 'j', "l", "m", "f", "d"]:
 											("generic", "generic", "none", "none", "none", "none"),
 											['PVA'+mang_type, 'P'+mang_type, mang_type],
 											["12memory_order", "12memory_order"])
+		gen_three_variants("atomic_compare_exchange_strong", SIG_TO_LLVM_TYPE_MAP['b'], '',
+											("private", "private", "none", "none", "none", "none"),
+											['PVA'+mang_type, 'P'+mang_type, mang_type],
+											["12memory_order", "12memory_order"])
+		gen_three_variants("atomic_compare_exchange_weak", SIG_TO_LLVM_TYPE_MAP['b'], '',
+											("private", "private", "none", "none", "none", "none"),
+											['PVA'+mang_type, 'P'+mang_type, mang_type],
+											["12memory_order", "12memory_order"])
+		gen_three_variants("atomic_compare_exchange_strong", SIG_TO_LLVM_TYPE_MAP['b'], '',
+											("generic", "private", "none", "none", "none", "none"),
+											['PVA'+mang_type, 'P'+mang_type, mang_type],
+											["12memory_order", "12memory_order"])
+		gen_three_variants("atomic_compare_exchange_weak", SIG_TO_LLVM_TYPE_MAP['b'], '',
+											("generic", "private", "none", "none", "none", "none"),
+											['PVA'+mang_type, 'P'+mang_type, mang_type],
+											["12memory_order", "12memory_order"])
 
 
 f = "atomic_flag_test_and_set"
