@@ -21,13 +21,12 @@
    THE SOFTWARE.
 */
 
-extern size_t _global_offset_x;
-extern size_t _global_offset_y;
-extern size_t _global_offset_z;
+extern const size_t _global_offset_x;
+extern const size_t _global_offset_y;
+extern const size_t _global_offset_z;
 
-__attribute__ ((noinline))
-size_t _CL_OVERLOADABLE
-get_global_offset(unsigned int dimindx)
+size_t _CL_OVERLOADABLE _CL_READNONE _CL_OPTNONE
+get_global_offset (unsigned int dimindx)
 {
   switch(dimindx)
     {
