@@ -21,17 +21,16 @@
    THE SOFTWARE.
 */
 
-extern size_t _local_size_x;
-extern size_t _local_size_y;
-extern size_t _local_size_z;
+extern const size_t _local_size_x;
+extern const size_t _local_size_y;
+extern const size_t _local_size_z;
 
-extern size_t _num_groups_x;
-extern size_t _num_groups_y;
-extern size_t _num_groups_z;
+extern const size_t _num_groups_x;
+extern const size_t _num_groups_y;
+extern const size_t _num_groups_z;
 
-__attribute__ ((noinline))
-size_t _CL_OVERLOADABLE
-get_global_size(unsigned int dimindx)
+size_t _CL_OVERLOADABLE _CL_READNONE _CL_OPTNONE
+get_global_size (unsigned int dimindx)
 {
   switch(dimindx)
     {
