@@ -26,32 +26,32 @@
 
 #ifndef __OPENCL_VERSION__
 
-typedef int int2 __attribute__ ((__ext_vector_type__ (2)));
-typedef int int3 __attribute__ ((__ext_vector_type__ (3)));
-typedef int int4 __attribute__ ((__ext_vector_type__ (4)));
-typedef int int8 __attribute__ ((__ext_vector_type__ (8)));
-typedef int int16 __attribute__ ((__ext_vector_type__ (16)));
+typedef int32_t int2 __attribute__ ((__ext_vector_type__ (2)));
+typedef int32_t int3 __attribute__ ((__ext_vector_type__ (3)));
+typedef int32_t int4 __attribute__ ((__ext_vector_type__ (4)));
+typedef int32_t int8 __attribute__ ((__ext_vector_type__ (8)));
+typedef int32_t int16 __attribute__ ((__ext_vector_type__ (16)));
 
-typedef unsigned int uint;
-typedef uint uint2 __attribute__ ((__ext_vector_type__ (2)));
-typedef uint uint3 __attribute__ ((__ext_vector_type__ (3)));
-typedef uint uint4 __attribute__ ((__ext_vector_type__ (4)));
-typedef uint uint8 __attribute__ ((__ext_vector_type__ (8)));
-typedef uint uint16 __attribute__ ((__ext_vector_type__ (16)));
+typedef uint32_t uint;
+typedef uint32_t uint2 __attribute__ ((__ext_vector_type__ (2)));
+typedef uint32_t uint3 __attribute__ ((__ext_vector_type__ (3)));
+typedef uint32_t uint4 __attribute__ ((__ext_vector_type__ (4)));
+typedef uint32_t uint8 __attribute__ ((__ext_vector_type__ (8)));
+typedef uint32_t uint16 __attribute__ ((__ext_vector_type__ (16)));
 
-typedef long long2 __attribute__ ((__ext_vector_type__ (2)));
-typedef long long3 __attribute__ ((__ext_vector_type__ (3)));
-typedef long long4 __attribute__ ((__ext_vector_type__ (4)));
-typedef long long8 __attribute__ ((__ext_vector_type__ (8)));
-typedef long long16 __attribute__ ((__ext_vector_type__ (16)));
+typedef int64_t long2 __attribute__ ((__ext_vector_type__ (2)));
+typedef int64_t long3 __attribute__ ((__ext_vector_type__ (3)));
+typedef int64_t long4 __attribute__ ((__ext_vector_type__ (4)));
+typedef int64_t long8 __attribute__ ((__ext_vector_type__ (8)));
+typedef int64_t long16 __attribute__ ((__ext_vector_type__ (16)));
 
-typedef unsigned long ulong;
+typedef uint64_t ulong;
 
-typedef ulong ulong2 __attribute__ ((__ext_vector_type__ (2)));
-typedef ulong ulong3 __attribute__ ((__ext_vector_type__ (3)));
-typedef ulong ulong4 __attribute__ ((__ext_vector_type__ (4)));
-typedef ulong ulong8 __attribute__ ((__ext_vector_type__ (8)));
-typedef ulong ulong16 __attribute__ ((__ext_vector_type__ (16)));
+typedef uint64_t ulong2 __attribute__ ((__ext_vector_type__ (2)));
+typedef uint64_t ulong3 __attribute__ ((__ext_vector_type__ (3)));
+typedef uint64_t ulong4 __attribute__ ((__ext_vector_type__ (4)));
+typedef uint64_t ulong8 __attribute__ ((__ext_vector_type__ (8)));
+typedef uint64_t ulong16 __attribute__ ((__ext_vector_type__ (16)));
 
 typedef float float2 __attribute__ ((__ext_vector_type__ (2)));
 typedef float float3 __attribute__ ((__ext_vector_type__ (3)));
