@@ -55,8 +55,12 @@ namespace pocl {
 
 typedef std::map<llvm::Function*, llvm::Function*> FunctionMapping;
 
+constexpr unsigned NumWorkgroupVariables = 18;
 extern const char *WorkgroupVariablesArray[];
 extern const std::vector<std::string> WorkgroupVariablesVector;
+constexpr unsigned NumWIFuncNames = 11;
+extern const char *WIFuncNameArray[];
+extern const std::vector<std::string> WIFuncNameVec;
 
 void regenerate_kernel_metadata(llvm::Module &M, FunctionMapping &kernels);
 
@@ -68,6 +72,7 @@ void eraseFunctionAndCallers(llvm::Function *Function);
 
 bool isAutomaticLocal(llvm::Function *F, llvm::GlobalVariable &Var);
 
+POCL_EXPORT
 bool isGVarUsedByFunction(llvm::GlobalVariable *GVar, llvm::Function *F);
 
 // Checks if the given argument of Func is a local buffer.
@@ -91,6 +96,17 @@ bool isKernelToProcess(const llvm::Function &F);
 
 llvm::Metadata *createConstantIntMD(llvm::LLVMContext &C, int32_t Val);
 
+// Fixes switch statements that have a default case that is a simple
+// unreachable instruction. LLVM does this as "optimization", however it breaks
+// the (post) dominator-tree analysis, because the unreachable instruction
+// creates an additional function exit path. PoCL's ImplicitConditionalBarriers
+// pass then erroneously adds barriers because it thinks some basic blocks are
+// inside conditional blocks, when they're not.
+// TODO this is fragile, LLVM can introduce more optimizations that create
+// unreachable blocks. However I couldn't find any working way to make PDT
+// ignore blocks with an unreachable inst.
+void removeUnreachableSwitchCases(llvm::Function &F);
+
 /**
  * \brief Clones a DISubprogram with changed function name and scope.
  *
@@ -121,17 +137,12 @@ void CloneFunctionIntoAbs(llvm::Function *NewFunc,
                           llvm::ValueMapTypeRemapper *TypeMapper = nullptr,
                           llvm::ValueMaterializer *Materializer = nullptr) {
 
-#if LLVM_MAJOR < 13
-  CloneFunctionInto(NewFunc, OldFunc, VMap, true, Returns, NameSuffix, CodeInfo,
-                    TypeMapper, Materializer);
-#else
                     // ClonedModule DifferentModule LocalChangesOnly
                     // GlobalChanges
   CloneFunctionInto(NewFunc, OldFunc, VMap,
                     (sameModule ? llvm::CloneFunctionChangeType::GlobalChanges
                                 : llvm::CloneFunctionChangeType::DifferentModule),
                     Returns, NameSuffix, CodeInfo, TypeMapper, Materializer);
-#endif
 }
 
 #if LLVM_MAJOR < 15
@@ -139,14 +150,6 @@ void CloneFunctionIntoAbs(llvm::Function *NewFunc,
 #define getValueType getType()->getElementType
 #endif /* LLVM_OPAQUE_POINTERS */
 
-#if LLVM_MAJOR < 14
-#define LLVMBuildGEP2(A, B, C, D, E, F) LLVMBuildGEP(A, C, D, E, F)
-#endif
-
-#if LLVM_MAJOR < 11
-#define CBS_NO_PHIS_IN_SPLIT
-#endif
-
 // macros for registering LLVM passes & analyses with old & new PM
 
 #define REGISTER_NEW_FPASS(PNAME, PCLASS, PDESC)                               \
