@@ -20,34 +20,51 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include <set>
-
 #ifndef POCL_LOOP_BARRIERS_H
 #define POCL_LOOP_BARRIERS_H
 
-#include "CompilerWarnings.h"
-IGNORE_COMPILER_WARNING("-Wunused-parameter")
+#include "config.h"
 
-#include "llvm/Analysis/LoopPass.h"
+#include <llvm/IR/Function.h>
+#include <llvm/IR/PassManager.h>
+#include <llvm/Pass.h>
+#include <llvm/Passes/PassBuilder.h>
 
-POP_COMPILER_DIAGS
+#if LLVM_MAJOR < MIN_LLVM_NEW_PASSMANAGER
+#include <llvm/Analysis/LoopPass.h>
+#endif
 
 namespace pocl {
-  class LoopBarriers : public llvm::LoopPass {
-    
-  public:
-    static char ID;
-    
-  LoopBarriers() : LoopPass(ID) {}
-    
-    virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const;
-    virtual bool runOnLoop(llvm::Loop *L, llvm::LPPassManager &LPM);
 
-  private:
-    llvm::DominatorTree *DT;
+#if LLVM_MAJOR < MIN_LLVM_NEW_PASSMANAGER
 
-    bool ProcessLoop(llvm::Loop *L, llvm::LPPassManager &LPM);
-  };
-}
+class LoopBarriers : public llvm::LoopPass
+{
+public:
+  static char ID;
+  LoopBarriers() : LoopPass(ID) {};
+  virtual ~LoopBarriers() {};
+
+  virtual bool runOnLoop(llvm::Loop *L, llvm::LPPassManager &LPM) override;
+  virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const override;
+};
+
+#else
+
+#include <llvm/Analysis/LoopAnalysisManager.h>
+#include <llvm/Transforms/Scalar/LoopPassManager.h>
+
+class LoopBarriers : public llvm::PassInfoMixin<LoopBarriers> {
+public:
+  static void registerWithPB(llvm::PassBuilder &B);
+  llvm::PreservedAnalyses run(llvm::Loop &L, llvm::LoopAnalysisManager &AM,
+                              llvm::LoopStandardAnalysisResults &AR,
+                              llvm::LPMUpdater &U);
+  static bool isRequired() { return true; }
+};
+
+#endif
+
+  } // namespace pocl
 
 #endif
