@@ -336,14 +336,14 @@ ParallelRegion::dumpNames()
 {
   for (iterator i = begin(), e = end(); i != e; ++i)
     {
-      std::cout << (*i)->getName().str();
-      if (entryBB() == (*i)) 
-        std::cout << "(EN)";
-      if (exitBB() == (*i))
-        std::cout << "(EX)";
-      std::cout << " ";
+    std::cerr << (*i)->getName().str();
+    if (entryBB() == (*i))
+      std::cerr << "(EN)";
+    if (exitBB() == (*i))
+      std::cerr << "(EX)";
+    std::cerr << " ";
     }
-  std::cout << std::endl;
+    std::cerr << std::endl;
 }
 
 ParallelRegion *
@@ -403,9 +403,9 @@ ParallelRegion::Verify()
           std::set<llvm::BasicBlock*> highlights;
           highlights.insert(entryBB());
           highlights.insert(*i);
-          pocl::dumpCFG(
-            *(*i)->getParent(), (*i)->getParent()->getName().str() + ".dot",
-            &prvec, &highlights);
+          pocl::dumpCFG(*(*i)->getParent(),
+                        (*i)->getParent()->getName().str() + ".dot", nullptr,
+                        &prvec, &highlights);
           assert(0 && "Incoming edges to non-entry block!");
           return false;
         } else if (!Barrier::hasBarrier(*ii)) {
@@ -431,7 +431,8 @@ ParallelRegion::Verify()
       highlights.insert(exitBB());
       exitBB()->dump();
       dumpNames();
-      dumpCFG(*(*i)->getParent(), "broken.dot", &regions, &highlights);
+      dumpCFG(*(*i)->getParent(), "broken.dot", nullptr,
+              &regions, &highlights);
 #endif
 
       assert(0 && "Multiple outgoing edges from exit block!");
@@ -504,16 +505,6 @@ ParallelRegion::AddParallelLoopMetadata(
         continue;
       }
 
-#if LLVM_VERSION_MAJOR < 13 &&                                                 \
-    !(LLVM_VERSION_MAJOR == 12 && LLVM_VERSION_MINOR >= 0 &&                   \
-      LLVM_VERSION_PATCH >= 1)
-      // This check will skip insertion of metadata on loads inside conditions
-      // before LLVM 12.0.1.
-      if (ii->mayReadFromMemory() && !IsLoadUnconditionallySafe(&*ii)) {
-        continue;
-      }
-#endif
-
       MDNode *NewMD = MDNode::get(bb->getContext(), Identifier);
       MDNode *OldMD = ii->getMetadata(PARALLEL_MD_NAME);
       if (OldMD != nullptr) {
@@ -633,7 +624,7 @@ ParallelRegion::LocalIDZLoad()
   return LocalIDZLoadInstr = builder.CreateLoad(
 #if LLVM_MAJOR > 14
              Ptr->getValueType(),
-#elif LLVM_MAJOR > 12
+#else
              Ptr->getType()->getPointerElementType(),
 #endif
              Ptr);
@@ -653,7 +644,7 @@ ParallelRegion::LocalIDYLoad()
   return LocalIDYLoadInstr = builder.CreateLoad(
 #if LLVM_MAJOR > 14
              Ptr->getValueType(),
-#elif LLVM_MAJOR > 12
+#else
              Ptr->getType()->getPointerElementType(),
 #endif
              Ptr);
@@ -673,7 +664,7 @@ ParallelRegion::LocalIDXLoad()
   return LocalIDXLoadInstr = builder.CreateLoad(
 #if LLVM_MAJOR > 14
              Ptr->getValueType(),
-#elif LLVM_MAJOR > 12
+#else
              Ptr->getType()->getPointerElementType(),
 #endif
              Ptr);
@@ -711,18 +702,11 @@ ParallelRegion::InjectPrintF
        /*Name=*/"printf", M); 
     printfFunc->setCallingConv(CallingConv::C);
 
-#if LLVM_MAJOR > 13
     AttributeList func_printf_PAL =
         AttributeList()
             .addAttributeAtIndex(M->getContext(), 1U, Attribute::NoCapture)
             .addAttributeAtIndex(M->getContext(), 4294967295U,
                                  Attribute::NoUnwind);
-#else
-    AttributeList func_printf_PAL =
-        AttributeList()
-            .addAttribute(M->getContext(), 1U, Attribute::NoCapture)
-            .addAttribute(M->getContext(), 4294967295U, Attribute::NoUnwind);
-#endif
 
     printfFunc->setAttributes(func_printf_PAL);
   }
