@@ -20,8 +20,8 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 // IN THE SOFTWARE.
 
-#ifndef _POCL_UNIFY_PRINTF_H
-#define _POCL_UNIFY_PRINTF_H
+#ifndef POCL_UNIFY_PRINTF_H
+#define POCL_UNIFY_PRINTF_H
 
 #include <llvm/IR/Module.h>
 
