@@ -2,13 +2,14 @@
  * \brief poclu_misc - misc generic OpenCL helper functions
 
    Copyright (c) 2013 Pekka Jääskeläinen / Tampere University of Technology
-   Copyright (c) 2014 Kalle Raiskila
+                 2014 Kalle Raiskila
+                 2023-2024 Pekka Jääskeläinen / Intel Finland Oy
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+ deal in the Software without restriction, including without limitation the
+ rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
 
    The above copyright notice and this permission notice shall be included in
@@ -18,9 +19,9 @@
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ IN THE SOFTWARE.
 
    \file
 */
@@ -35,19 +36,16 @@
 #include "pocl_opencl.h"
 
 cl_context
-poclu_create_any_context ()
+poclu_create_any_context2 (cl_platform_id *platform)
 {
   cl_uint i;
-  cl_platform_id platform;
 
-  clGetPlatformIDs (1, &platform, &i);
+  clGetPlatformIDs (1, platform, &i);
   if (i == 0)
     return (cl_context) 0;
 
-  cl_context_properties properties[] =
-    {CL_CONTEXT_PLATFORM,
-     (cl_context_properties)platform,
-     0};
+  cl_context_properties properties[]
+    = { CL_CONTEXT_PLATFORM, (cl_context_properties)(*platform), 0 };
 
   // create the OpenCL context on any available OCL device
   cl_context context = clCreateContextFromType (properties,
@@ -57,6 +55,13 @@ poclu_create_any_context ()
   return context;
 }
 
+cl_context
+poclu_create_any_context ()
+{
+  cl_platform_id platform = NULL;
+  return poclu_create_any_context2 (&platform);
+}
+
 cl_int
 poclu_get_any_device (cl_context *context, cl_device_id *device,
                       cl_command_queue *queue)
@@ -496,7 +501,7 @@ pocl_getpath (char *path, size_t len, const char *explicit_binary,
 int
 poclu_load_program_multidev (cl_context context, cl_device_id *devices,
                              cl_uint num_devices, const char *basename,
-                             int spir, int spirv, int poclbin,
+                             int spirv, int poclbin,
                              const char *explicit_binary,
                              const char *extra_build_opts, cl_program *p)
 {
@@ -511,7 +516,7 @@ poclu_load_program_multidev (cl_context context, cl_device_id *devices,
   cl_program program;
   cl_int err = CL_SUCCESS;
 
-  int from_source = (!spir && !spirv && !poclbin);
+  int from_source = (!spirv && !poclbin);
   TEST_ASSERT (num_devices > 0);
   cl_device_id device = devices[0];
   if (!from_source)
@@ -550,21 +555,18 @@ poclu_load_program_multidev (cl_context context, cl_device_id *devices,
   *p = NULL;
   final_opts[0] = 0;
   if (extra_build_opts != NULL)
-    strcat(final_opts, extra_build_opts);
+    strncat(final_opts, extra_build_opts, 2047);
 
-  if (spir || spirv)
+  if (spirv)
     {
       TEST_ASSERT (device != NULL);
-      strcat (final_opts, " -x spir -spir-std=1.2");
       err = clGetDeviceInfo (device, CL_DEVICE_EXTENSIONS, 1024, extensions,
                              NULL);
       CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo extensions");
 
-      if ((spir && strstr (extensions, "cl_khr_spir") == NULL)
-          || (spirv && strstr (extensions, "cl_khr_il_program") == NULL))
+      if (spirv && strstr (extensions, "cl_khr_il_program") == NULL)
         {
-          printf ("SPIR%s not supported, cannot run the test\n",
-                  spirv ? "-V" : "");
+          printf ("SPIR-V not supported, cannot run the test\n");
           return -1;
         }
 
@@ -575,7 +577,7 @@ poclu_load_program_multidev (cl_context context, cl_device_id *devices,
       if (little_endian == CL_FALSE)
         {
           fprintf (stderr,
-                   "SPIR can only be tested on little-endian devices\n");
+                   "SPIR-V can only be tested on little-endian devices\n");
           return 1;
         }
 
@@ -583,20 +585,10 @@ poclu_load_program_multidev (cl_context context, cl_device_id *devices,
                              &address_bits, NULL);
       CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo addr bits");
 
-      if (spirv)
-      {
-        if (address_bits < 64)
-          ext = ".spirv32";
-        else
-          ext = ".spirv64";
-      }
+      if (address_bits < 64)
+        ext = ".spirv32";
       else
-      {
-        if (address_bits < 64)
-          ext = ".spir32";
-        else
-          ext = ".spir64";
-      }
+        ext = ".spirv64";
     }
 
   if (poclbin)
@@ -667,11 +659,11 @@ poclu_load_program_multidev (cl_context context, cl_device_id *devices,
 
 int
 poclu_load_program (cl_context context, cl_device_id device,
-                    const char *basename, int spir, int spirv, int poclbin,
+                    const char *basename, int spirv, int poclbin,
                     const char *explicit_binary, const char *extra_build_opts,
                     cl_program *p)
 {
-  return poclu_load_program_multidev (context, &device, 1, basename, spir,
+  return poclu_load_program_multidev (context, &device, 1, basename,
                                       spirv, poclbin, explicit_binary,
                                       extra_build_opts, p);
 }
