@@ -67,9 +67,9 @@ main(void)
         R[i] = 0;
     }
 
+    std::vector<cl::Platform> platformList;
+    bool ok = false;
     try {
-        std::vector<cl::Platform> platformList;
-
         // Pick platform
         cl::Platform::get(&platformList);
 
@@ -128,7 +128,7 @@ main(void)
             0,
             WORK_ITEMS * sizeof(int));
 
-        bool ok = true;
+        ok = true;
         for (int i = 0; i < WORK_ITEMS; i++) {
 
             if ((int)R[i] != i + 1) {
@@ -145,22 +145,19 @@ main(void)
             (void *) output);
 
         queue.finish();
-        platformList[0].unloadCompiler();
-
-        if (ok) {
-            std::cout << "OK" << std::endl;
-            return EXIT_SUCCESS;
-        }
     }
     catch (cl::Error &err) {
-         std::cerr
-             << "ERROR: "
-             << err.what()
-             << "("
-             << err.err()
-             << ")"
-             << std::endl;
+        std::cerr << "ERROR: " << err.what() << "(" << err.err() << ")"
+                  << std::endl;
+        return EXIT_FAILURE;
     }
+    platformList[0].unloadCompiler();
 
-    return EXIT_FAILURE;
+    if (ok) {
+        std::cout << "OK" << std::endl;
+        return EXIT_SUCCESS;
+    } else {
+        std::cout << "FAIL" << std::endl;
+        return EXIT_FAILURE;
+    }
 }
