@@ -61,10 +61,10 @@ int
 main(void)
 {
     float A[BUFFER_SIZE];
-    bool success;
+    bool success = false;
 
+    std::vector<cl::Platform> platformList;
     try {
-        std::vector<cl::Platform> platformList;
 
         // Pick platform
         cl::Platform::get(&platformList);
@@ -136,22 +136,20 @@ main(void)
         queue.enqueueUnmapMemObject(
             aBuffer, (void *) res);
         queue.finish();
-        platformList[0].unloadCompiler();
-
-        if (success) {
-            std::cout << "OK" << std::endl;
-            return EXIT_SUCCESS;
-        }
     }
     catch (cl::Error &err) {
-         std::cerr
-             << "ERROR: "
-             << err.what()
-             << "("
-             << err.err()
-             << ")"
-             << std::endl;
+        std::cerr << "ERROR: " << err.what() << "(" << err.err() << ")"
+                  << std::endl;
+        return EXIT_FAILURE;
     }
 
-    return EXIT_FAILURE;
+    platformList[0].unloadCompiler();
+
+    if (success) {
+        std::cout << "OK" << std::endl;
+        return EXIT_SUCCESS;
+    } else {
+        std::cout << "FAIL\n";
+        return EXIT_FAILURE;
+    }
 }
