@@ -70,8 +70,9 @@ main(void)
         R[i] = i;
     }
 
+    std::vector<cl::Platform> platformList;
+    bool ok = false;
     try {
-        std::vector<cl::Platform> platformList;
 
         // Pick platform
         cl::Platform::get(&platformList);
@@ -133,7 +134,7 @@ main(void)
             0,
             WORK_ITEMS * sizeof(int));
 
-        bool ok = true;
+        ok = true;
         // TODO: validate results
         for (int gid = 0; gid < WORK_ITEMS; gid++) {
 
@@ -159,22 +160,20 @@ main(void)
             (void *) output);
 
         queue.finish();
-        platformList[0].unloadCompiler();
-
-        if (ok) {
-            std::cout << "OK" << std::endl;
-            return EXIT_SUCCESS;
-        }
     }
     catch (cl::Error &err) {
-         std::cerr
-             << "ERROR: "
-             << err.what()
-             << "("
-             << err.err()
-             << ")"
-             << std::endl;
+        std::cerr << "ERROR: " << err.what() << "(" << err.err() << ")"
+                  << std::endl;
+        return EXIT_FAILURE;
     }
 
-    return EXIT_FAILURE;
+    platformList[0].unloadCompiler();
+
+    if (ok) {
+        std::cout << "OK" << std::endl;
+        return EXIT_SUCCESS;
+    } else {
+        std::cout << "FAIL\n";
+        return EXIT_FAILURE;
+    }
 }
