@@ -59,8 +59,9 @@ main(void)
         in[i] = (float)rand()/(float)RAND_MAX;;
     }
 
+    std::vector<cl::Platform> platformList;
+    bool ok = false;
     try {
-        std::vector<cl::Platform> platformList;
 
         // Pick platform
         cl::Platform::get(&platformList);
@@ -121,7 +122,7 @@ main(void)
             0,
             BUFFER_SIZE * sizeof(float));
 
-        bool ok = true;
+        ok = true;
         for (int i = 0; i < BUFFER_SIZE; i++) {
             if (   (in[i] >  0.5f && out[i] < 1.0f)
                 || (in[i] <= 0.5f && out[i] > 0.0f))
@@ -134,22 +135,20 @@ main(void)
             (void *) output);
 
         queue.finish();
-        platformList[0].unloadCompiler();
-
-        if (ok) {
-            std::cout << "OK" << std::endl;
-            return EXIT_SUCCESS;
-        }
     }
     catch (cl::Error &err) {
-         std::cerr
-             << "ERROR: "
-             << err.what()
-             << "("
-             << err.err()
-             << ")"
-             << std::endl;
+        std::cerr << "ERROR: " << err.what() << "(" << err.err() << ")"
+                  << std::endl;
+        return EXIT_FAILURE;
     }
 
-    return EXIT_FAILURE;
+    platformList[0].unloadCompiler();
+
+    if (ok) {
+        std::cout << "OK" << std::endl;
+        return EXIT_SUCCESS;
+    } else {
+        std::cout << "FAIL\n";
+        return EXIT_FAILURE;
+    }
 }
