@@ -70,8 +70,8 @@ main (int argc, char **argv)
   cl_kernel kernel_a = NULL, kernel_b = NULL, kernel_c = NULL;
   const char *kernel_buffer = NULL;
 
-  err = poclu_get_multiple_devices (&platform, &context, &num_devices,
-                                    &devices, &queues);
+  err = poclu_get_multiple_devices (&platform, &context, 0, &num_devices,
+                                    &devices, &queues, 0);
   CHECK_OPENCL_ERROR_IN ("poclu_get_multiple_devices");
 
   printf ("NUM DEVICES: %u \n", num_devices);
