@@ -62,11 +62,10 @@ main(void)
           CHECK_CL_ERROR(clReleaseCommandQueue(queue));
 
           CHECK_CL_ERROR (clReleaseContext (context));
+          CHECK_CL_ERROR (clUnloadPlatformCompiler (platforms[i]));
         }
     }
 
-  CHECK_CL_ERROR (clUnloadCompiler ());
-
   printf ("OK\n");
   return EXIT_SUCCESS;
 }
