@@ -372,16 +372,17 @@ int main()
 {
   cl_int err;
 
-  cl_context ctx;
-  cl_device_id did;
-  cl_command_queue queue;
+  cl_platform_id pid = NULL;
+  cl_context ctx = NULL;
+  cl_device_id did = NULL;
+  cl_command_queue queue = NULL;
 
   size_t program_size;
   char* program_buffer;
 
   cl_program program = NULL;
 
-  poclu_get_any_device(&ctx, &did, &queue);
+  poclu_get_any_device2 (&ctx, &did, &queue, &pid);
   TEST_ASSERT(ctx);
   TEST_ASSERT(did);
   TEST_ASSERT(queue);
@@ -407,44 +408,10 @@ int main()
   printf("\nNON-SPIR\n");
   TEST_ASSERT(test_program(program, 0) == EXIT_SUCCESS);
 
-  CHECK_CL_ERROR(clReleaseProgram(program));
-
-  char extensions[1024];
-  err = clGetDeviceInfo(did, CL_DEVICE_EXTENSIONS, 1024, extensions, NULL);
-  CHECK_OPENCL_ERROR_IN("clGetDeviceInfo");
-  if (strstr(extensions, "cl_khr_spir") == NULL)
-    {
-      printf ("SPIR not supported, skipping SPIR arg info tests\n");
-      goto FINISH;
-    }
-
-  /* SPIR program */
-
-  printf ("\nSPIR with metadata\n");
-
-  const char *filename = (address_bits == 32 ? meta_files[0] : meta_files[1]);
-  TEST_ASSERT (spir_program (filename, ctx, did, &program) == EXIT_SUCCESS);
-
-  TEST_ASSERT (test_program (program, 1) == EXIT_SUCCESS);
-
-  CHECK_CL_ERROR(clReleaseProgram(program));
-
-  /* SPIR program without metadata - currently disabled since Clang seems to
-   * always generate metadata. */
-  /*
-    printf("\nSPIR WITHOUT metadata\n");
-    filename = (address_bits == 32 ? nometa_files[0] : nometa_files[1]);
-    TEST_ASSERT(spir_program(filename, ctx, did, &program) == EXIT_SUCCESS);
-
-    TEST_ASSERT(test_program_nometa(program) == EXIT_SUCCESS);
-
-    CHECK_CL_ERROR(clReleaseProgram(program));
-  */
-
-FINISH:
+  CHECK_CL_ERROR (clReleaseProgram (program));
   CHECK_CL_ERROR (clReleaseCommandQueue (queue));
   CHECK_CL_ERROR (clReleaseContext (ctx));
-  CHECK_CL_ERROR (clUnloadCompiler ());
+  CHECK_CL_ERROR (clUnloadPlatformCompiler (pid));
 
   printf("\nOK\n");
   return EXIT_SUCCESS;
