@@ -44,7 +44,7 @@ main(void)
 
   CHECK_CL_ERROR (clReleaseMemObject (mem));
   CHECK_CL_ERROR (clReleaseContext (context));
-  CHECK_CL_ERROR (clUnloadCompiler ());
+  CHECK_CL_ERROR (clUnloadPlatformCompiler (platform[0]));
 
   return EXIT_SUCCESS;
 }
