@@ -30,18 +30,18 @@
 int main(int argc, char **argv)
 {
   cl_int err;
-  cl_context ctx;
-  cl_command_queue queue;
-  cl_device_id did;
   cl_bool has_img = CL_FALSE;
 
+  cl_platform_id pid = NULL;
+  cl_context ctx = NULL;
+  cl_device_id did = NULL;
+  cl_command_queue queue = NULL;
+
   const size_t buf_size = sizeof(cl_int);
 
   cl_mem buf, img;
 
-
-
-  poclu_get_any_device(&ctx, &did, &queue);
+  poclu_get_any_device2 (&ctx, &did, &queue, &pid);
   TEST_ASSERT(ctx);
   TEST_ASSERT(did);
   TEST_ASSERT(queue);
@@ -227,8 +227,7 @@ int main(int argc, char **argv)
 
   CHECK_CL_ERROR (clReleaseCommandQueue (queue));
   CHECK_CL_ERROR (clReleaseContext (ctx));
-
-  CHECK_CL_ERROR (clUnloadCompiler ());
+  CHECK_CL_ERROR (clUnloadPlatformCompiler (pid));
 
   printf ("OK\n");
   return EXIT_SUCCESS;
