@@ -10,7 +10,7 @@ SCRIPTPATH=$( realpath "$0"  )
 RELPATH=$(dirname "$SCRIPTPATH")
 
 # cd to root directory of the git repo
-pushd ${GITROOT} > /dev/null
+pushd "${GITROOT}" > /dev/null || exit 1
 
 case "$(git describe --always --dirty=-DIRTY)" in
   *-DIRTY)
@@ -19,12 +19,12 @@ case "$(git describe --always --dirty=-DIRTY)" in
 esac
 
 PATCHY=$(mktemp /tmp/pocl.XXXXXXXX.patch)
-trap "rm -f $PATCHY" EXIT
+trap 'rm -f $PATCHY' EXIT
 
-git show -U0 --no-color >$PATCHY
+git show -U0 --no-color >"$PATCHY"
 
-$RELPATH/clang-format-diff.py -regex '.*(\.h$|\.c$|\.cl$)' -i -p1 -style GNU <$PATCHY
-$RELPATH/clang-format-diff.py -regex '(.*(\.hpp$|\.hh$|\.cc$|\.cpp$))|(lib/llvmopencl/.*)|(lib/CL/devices/tce/.*)' -i -p1 -style LLVM <$PATCHY
+"$RELPATH"/clang-format-diff.py -regex '.*(\.h$|\.c$|\.cl$)' -i -p1 -style=file:"$RELPATH/style.GNU" <"$PATCHY"
+"$RELPATH"/clang-format-diff.py -regex '(.*(\.hpp$|\.hh$|\.cc$|\.cpp$))|(lib/llvmopencl/.*)|(lib/CL/devices/tce/.*)' -i -p1 -style LLVM <"$PATCHY"
 
 if [ -z "$(git diff)" ]; then
   echo "No changes."
@@ -32,4 +32,4 @@ if [ -z "$(git diff)" ]; then
 fi
 
 # cd back whence we were previously
-popd > /dev/null
+popd > /dev/null || exit 1
